@@ -63,7 +63,7 @@ blacklist {
     # Note: On sles11/rhel6, multipath failed if no multipath device is found.
     while ! multipath ; do
         echo
-        choice="$( UserInput -I MULTIPATH_FAILED_TO_LIST_DEVICE -p "$prompt" -D "${choices[0]}" "${choices[@]}")"&& wilful_input="yes" || wilful_input="no"
+        choice="$( UserInput -I MULTIPATH_FAILED_TO_LIST_DEVICE -p "$prompt" -D "${choices[0]}" "${choices[@]}")" && wilful_input="yes" || wilful_input="no"
         case "$choice" in
             (${choices[0]})
                 # continue recovery without multipath
@@ -119,9 +119,41 @@ fi
 
 ### Create multipath devices (at least partitions on them).
 function create_multipath() {
-    local device=$1
-    if grep "^multipath $device " "$LAYOUT_FILE" 1>&2 ; then
-        Log "Found current or former multipath device $device in $LAYOUT_FILE: Creating partitions on it"
-        create_partitions "$device"
-    fi
+    local component device size label junk
+    read component device size label junk < <(grep "^multipath $1 " "$LAYOUT_FILE")
+    [[ "$device" ]] || return 0
+
+    Log "Found current or former multipath device $device in $LAYOUT_FILE: Creating partitions on it"
+
+    # A Multipath device is a standard disk, but we cannot reuse the
+    # create_disk() code because it's not a block device *and* we don't
+    # have the 'disk' entry.
+
+    cat >> "$LAYOUT_CODE" <<EOF
+
+#
+# Code handling Multipath Device '$device'
+#
+
+Log "Erasing MBR of device $device"
+dd if=/dev/zero of=$device bs=512 count=1
+sync
+
+EOF
+
+    create_partitions "$device" "$label"
+
+    cat >> "$LAYOUT_CODE" <<EOF
+# Make sure device nodes are visible (eg. in RHEL4)
+my_udevtrigger
+my_udevsettle
+
+# Clean up transient partitions and resize shrinked ones
+delete_dummy_partitions_and_resize_real_ones
+
+#
+# End of code handling Multipath Device '$device'
+#
+
+EOF
 }
