@@ -110,7 +110,16 @@ done
 # to current block devices in the currently running recovery system:
 while read keyword orig_device orig_size junk ; do
     # Continue with next original device when it is already used as source in the mapping file:
-    is_mapping_source "$orig_device" && continue
+    if is_mapping_source "$orig_device" ; then
+        DebugPrint "Skip automapping $orig_device (already exists as source in $MAPPING_FILE)"
+        continue
+    fi
+    # The original device is not yet mapped (i.e. not used as source in the mapping file) so it needs to be mapped.
+    # Remember when target devices get known by the "same name and same size" tests
+    # that they cannot be used for recreating the current original device
+    # to avoid that already excluded target devices get needlessly
+    # considered again during the subsequent "same size" tests:
+    excluded_target_device_names=()
     # First, try to find if there is a current disk with same name and same size as the original:
     sysfs_device_name="$( get_sysfs_name "$orig_device" )"
     current_device="/sys/block/$sysfs_device_name"
@@ -119,20 +128,33 @@ while read keyword orig_device orig_size junk ; do
         # The current_device (e.g. /sys/block/sda) is not a block device so that
         # its matching actual block device (e.g. /dev/sda) must be determined:
         preferred_target_device_name="$( get_device_name $current_device )"
-        # Continue with next one if the current one is already used as target in the mapping file:
-        is_mapping_target "$preferred_target_device_name" && continue
         # Use the current one if it is of same size as the old one:
         if test "$orig_size" -eq "$current_size" ; then
-            # Ensure the determined target device is really a block device:
+            # Ensure the target device is really a block device on the replacement hardware.
+            # Here the target device has same name as the original device which was a block device on the original hardware
+            # but it might perhaps happen that this device name is not a block device on the replacement hardware:
             if test -b "$preferred_target_device_name" ; then
-                add_mapping "$orig_device" "$preferred_target_device_name"
-                LogPrint "Using $preferred_target_device_name (same name and same size) for recreating $orig_device"
-                # Continue with next original device in the LAYOUT_FILE:
-                continue
+                # Do not map if the current one is already used as target in the mapping file:
+                if is_mapping_target "$preferred_target_device_name" ; then
+                    DebugPrint "Cannot use $preferred_target_device_name (same name and same size) for recreating $orig_device ($preferred_target_device_name already exists as target in $MAPPING_FILE)"
+                    excluded_target_device_names+=( "$preferred_target_device_name" )
+                else
+                    # Ensure the determined target device is not write-protected:
+                    if is_write_protected "$preferred_target_device_name" ; then
+                        DebugPrint "Cannot use $preferred_target_device_name (same name and same size) for recreating $orig_device ($preferred_target_device_name is write-protected)"
+                        excluded_target_device_names+=( "$preferred_target_device_name" )
+                    else
+                        add_mapping "$orig_device" "$preferred_target_device_name"
+                        LogPrint "Using $preferred_target_device_name (same name and same size $current_size) for recreating $orig_device"
+                        # Continue with next original device because the current one is now mapped:
+                        continue
+                    fi
+                fi
             fi
         fi
     fi
-    # Else, loop over all current block devices to find one of the same size as the original:
+    # If there is no current disk with same name and same size as the original
+    # loop over all current block devices to find one of same size as the original:
     for current_device_path in /sys/block/* ; do
         # Continue with next block device if the current one has no queue directory:
         test -d $current_device_path/queue || continue
@@ -143,20 +165,33 @@ while read keyword orig_device orig_size junk ; do
         # The current_device_path (e.g. /sys/block/sdb) is not a block device so that
         # its matching actual block device (e.g. /dev/sdb) must be determined:
         preferred_target_device_name="$( get_device_name $current_device_path )"
-        # Continue with next one if the current one is already used as target in the mapping file:
-        is_mapping_target "$preferred_target_device_name" && continue
-        # Use the current one if it is of same size as the old one:
-        if test "$orig_size" -eq "$current_size" ; then
-            # Ensure the determined target device is really a block device:
-            if test -b "$preferred_target_device_name" ; then
-                add_mapping "$orig_device" "$preferred_target_device_name"
-                LogPrint "Using $preferred_target_device_name (same size) for recreating $orig_device"
-                # Break looping over all current block devices to find one
-                # and continue with next original device in the LAYOUT_FILE:
-                break
-            fi
+        # Ensure the determined target device is really a block device (cf. above):
+        test -b "$preferred_target_device_name" || continue
+        # Continue with next block device if the current one is not of same size as the original:
+        test "$orig_size" -eq "$current_size" || continue
+        # Continue with next block device if the current one was already excluded by the "same name and same size" tests above:
+        IsInArray "$preferred_target_device_name" "${excluded_target_device_names[@]}" && continue
+        # Continue with next block device if the current one is already used as target in the mapping file:
+        if is_mapping_target "$preferred_target_device_name" ; then
+            DebugPrint "Cannot use $preferred_target_device_name (same size) for recreating $orig_device ($preferred_target_device_name already exists as target in $MAPPING_FILE)"
+            continue
         fi
+        # Ensure the determined target device is not write-protected (cf. above):
+        if is_write_protected "$preferred_target_device_name" ; then
+            DebugPrint "Cannot use $preferred_target_device_name (same size) for recreating $orig_device ($preferred_target_device_name is write-protected)"
+            continue
+        fi
+        # The first of all current block devices with same size as the original that is not yet used as target gets used:
+        add_mapping "$orig_device" "$preferred_target_device_name"
+        LogPrint "Using $preferred_target_device_name (same size $current_size) for recreating $orig_device"
+        # Continue the outer while loop with next original device because the current one is now mapped:
+        continue 2
     done
+    # The original device could not be automapped because there is
+    # neither a current disk with same name and same size as the original
+    # nor is there a current disk with different name but same size as the original
+    # so the user must maually specify the right mapping target:
+    DebugPrint "Could not automap $orig_device (no disk with same size $orig_size found)"
 done < <( grep -E "^disk |^multipath " "$LAYOUT_FILE" )
 
 # For every unmapped original 'disk' device and 'multipath' device in the LAYOUT_FILE
@@ -173,7 +208,14 @@ while read keyword orig_device orig_size junk ; do
     # Loop over all current block devices to find appropriate ones wherefrom the user can choose:
     for current_device_path in /sys/block/* ; do
         current_device_basename="${current_device_path##*/}"
-        # Do not include removable devices in the choices for the user:
+        # Do not include removable devices in the choices for the user
+        # for example CDROM is removable because /sys/block/sr0/removable contains '1'
+        # but a USB disk is not removable because /sys/block/sdb/removable contains '0'
+        # so this condition is primarily there to skip CDROM devices
+        # (in particular the device where the ReaR recovery system was booted from)
+        # because we cannot test /sys/block/sr0/ro which usually contains '0'
+        # because that is usually a CD/DVD-RW device that can write (depending on the medium)
+        # cf. https://unix.stackexchange.com/questions/22019/how-can-i-test-whether-a-block-device-is-read-only-from-sys-or-proc
         if test "$( < $current_device_path/removable )" = "1" ; then
             Log "$current_device_basename excluded from device mapping choices (is a removable device)"
             continue
@@ -201,8 +243,13 @@ while read keyword orig_device orig_size junk ; do
             Log "$preferred_target_device_name excluded from device mapping choices (is already used as mapping target)"
             continue
         fi
+        # Continue with next block device if the current one is designated as write-protected:
+        if is_write_protected "$preferred_target_device_name"; then
+            Log "$preferred_target_device_name excluded from device mapping choices (is designated as write-protected)"
+            continue
+        fi
         # Add the current device as possible choice for the user:
-        possible_targets=( "${possible_targets[@]}" "$preferred_target_device_name" )
+        possible_targets+=( "$preferred_target_device_name" )
     done
     # Continue with next original device when no appropriate current block device is found where to it could be mapped:
     if ! test "${possible_targets[*]}" ; then
@@ -213,7 +260,7 @@ while read keyword orig_device orig_size junk ; do
     # At the end the mapping file is shown and the user can edit it if he does not like an automated mapping:
     if test "1" -eq "${#possible_targets[@]}" ; then
         add_mapping "$orig_device" "$possible_targets"
-        LogPrint "Using $possible_targets (the only appropriate) for recreating $orig_device"
+        LogPrint "Using $possible_targets (the only available of the disks) for recreating $orig_device"
         # Continue with next original device in the LAYOUT_FILE:
         continue
     fi
