@@ -322,7 +322,11 @@ function DoExitTasks () {
     done
 }
 
-# The command (actually the function) DoExitTasks is executed on exit from the shell:
+# The command (actually the function) DoExitTasks is executed on exit from the shell.
+# Avoid SC2218 "This function is only defined later. Move the definition up."
+# because it seems ShellCheck falsely thinks 'trap' is the below defined function
+# (i.e. it seems ShellCheck does not recognize 'builtin')
+# shellcheck disable=SC2218
 builtin trap "DoExitTasks" EXIT
 
 # Prepare that STDIN STDOUT and STDERR can be later redirected to anywhere
@@ -360,12 +364,16 @@ QuietAddExitTask "(( EXIT_FAIL_MESSAGE )) && echo '${MESSAGE_PREFIX}$PROGRAM $WO
 
 # USR1 is used to abort on errors.
 # It is not using PrintError but does direct output to the original STDERR.
-# Set EXIT_FAIL_MESSAGE to 0 to avoid an additional failed message via the QuietAddExitTask above:
+# Set EXIT_FAIL_MESSAGE to 0 to avoid an additional failed message via the QuietAddExitTask above.
+# Avoid SC2218 "This function is only defined later. Move the definition up."
+# because it seems ShellCheck falsely thinks 'trap' is the below defined function
+# (i.e. it seems ShellCheck does not recognize 'builtin')
+# shellcheck disable=SC2218
 builtin trap "EXIT_FAIL_MESSAGE=0 ; echo '${MESSAGE_PREFIX}Aborting due to an error, check $RUNTIME_LOGFILE for details' 1>&8 ; kill $MASTER_PID" USR1
 
 # Make sure nobody else can use trap:
 function trap () {
-    BugError "Forbidden usage of trap with '$@'. Use AddExitTask instead."
+    BugError "Forbidden usage of trap with '$*'. Use AddExitTask instead."
 }
 
 # For actually intended user messages output to the original STDOUT
@@ -398,15 +406,49 @@ function PrintError () {
 # For messages that should only appear in the log file output to the current STDERR
 # because (usually) the current STDERR is redirected to the log file:
 function Log () {
+    # RUNTIME_LOGFILE does not yet exists in case of early Error() in usr/sbin/rear
+    test -w "$RUNTIME_LOGFILE" || return 0
     # Have a timestamp with nanoseconds precision in any case
     # so that any subsequent Log() calls get logged with precise timestamps:
     { local timestamp=$( date +"%Y-%m-%d %H:%M:%S.%N " )
+      local prefix="${MESSAGE_PREFIX}${timestamp}"
+      # prefix_blanks has the printable characters in prefix replaced with blanks for indentation:
+      local prefix_blanks="$( tr '[:print:]' ' ' <<<"$prefix" )"
+      local message=""
       local log_message=""
-      if test $# -gt 0 ; then
-          log_message="${MESSAGE_PREFIX}${timestamp}$*"
-      else
-          log_message="${MESSAGE_PREFIX}${timestamp}$( cat )"
-      fi
+      test $# -gt 0 && message="$*" || message="$( cat )"
+      # The first line of message is prefixed with MESSAGE_PREFIX and timestamp
+      # and all subsequent lines in message are indented by prefix_blanks
+      # via bash parameter expansion ${message//$LF/$LF$prefix_blanks}
+      #   ${...}            - interpret ... using parameter expansion
+      #   message           - name of the variable containing the content
+      #   //...             - replace all instances of ...
+      #   $LF               - the literal newline character (see 'LF' above)
+      #   /...              - replace with ...
+      #   $LF$prefix_blanks - the literal newline character followed by the indentation blanks
+      # cf. https://superuser.com/questions/955935/how-can-i-replace-a-newline-with-its-escape-sequence
+      # that uses the literal newline character inline as in ${...//$'\n'/...}
+      # but that results partially wrong parameter expansion with bash version 3.1.17 in SLES10
+      # that seems to get somehow confused by the single quotes within parameter expansion:
+      #   # MESSAGE_PREFIX="message prefix "
+      #   # timestamp=$( date +"%Y-%m-%d %H:%M:%S.%N " )
+      #   # message="$( echo -e 'fist line\nsecond line\nthird line')"
+      #   # prefix="${MESSAGE_PREFIX}${timestamp}"
+      #   # prefix_blanks="$( tr '[:print:]' ' ' <<<"$prefix" )"
+      #   # log_message="${MESSAGE_PREFIX}${timestamp}${message//$'\n'/$'\n'$prefix_blanks}"
+      #   # echo "$log_message"
+      #   message prefix 2021-06-24 10:49:39.824719000 fist line'
+      #   '                                             second line'
+      #   '                                             third line
+      # so we use the LF variable (cf. how LF is set above)
+      #   # LF=$'\n'
+      #   # log_message="${MESSAGE_PREFIX}${timestamp}${message//$LF/$LF$prefix_blanks}"
+      #   # echo "$log_message"
+      #   message prefix 2021-06-24 10:49:39.824719000 fist line
+      #                                                second line
+      #                                                third line
+      # to make that parameter expansion also works with bash version 3.1.17 in SLES10:
+      log_message="${MESSAGE_PREFIX}${timestamp}${message//$LF/$LF$prefix_blanks}"
     } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
     # Append the log message explicitly to the log file to ensure that intended log messages
     # actually appear in the log file even inside { ... } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
@@ -447,9 +489,16 @@ function LogUserOutput () {
 # LogPrintError does not error out (the Error function is meant to error out).
 # LogPrintError is meant to show error messages when we do not want to error out,
 # (for example when at the end of "rear recover" it failed to install a bootloader).
+# Real error messages should be prefixed with 'Error: ' in the LogPrintError message.
 # LogPrintError is also meant to show important "error-like" messages to the user
 # (for example when the user must decide if that means a real error in his case)
-# and other important messages that must appear on the user's terminal
+# and other important messages that must appear on the user's terminal.
+# In particular when there was already a LogPrintError message
+# subsequent messages that are related to this LogPrintError message
+# must also be shown to the user as LogPrintError messages
+# to ensure the user gets them on his terminal regardless
+# whether or not he launched 'rear' in verbose mode.
+# Messages that are no real error messages should not be prefixed with 'Warning: '
 # cf. https://blog.schlomo.schapiro.org/2015/04/warning-is-waste-of-my-time.html
 function LogPrintError () {
     Log "$@"
@@ -468,7 +517,7 @@ function LogToSyslog () {
 # an alias, builtin, function, or a disk file that would be executed
 # see https://github.com/rear/rear/issues/729
 function has_binary () {
-    for bin in $@ ; do
+    for bin in "$@" ; do
         # Suppress success output via stdout which is crucial when has_binary is called
         # in other functions that provide their intended function results via stdout
         # to not pollute intended function results with intermixed has_binary stdout
@@ -529,6 +578,9 @@ function CallerSource () {
 }
 
 # Error exit:
+# It is an Error when the cause is not in ReaR's code
+# for example when the user specified something wrong
+# or when a called program exits with a fatal error.
 function Error () {
     # Get the last sourced script out of the log file:
     # Using the CallerSource function is not sufficient here because CallerSource results
@@ -539,22 +591,23 @@ function Error () {
     # but also the outdated scripts with leading 2-digit number get sourced
     # see the SourceStage function in lib/framework-functions.sh
     # so that we grep for script files names with two or more leading numbers:
-    { local last_sourced_script_log_entry=( $( grep -o ' Including .*/[0-9][0-9].*\.sh' $RUNTIME_LOGFILE | tail -n 1 ) )
-      # The last_sourced_script_log_entry contains: Including sub-path/to/script_file_name.sh
-      local last_sourced_script_sub_path="${last_sourced_script_log_entry[1]}"
-      local last_sourced_script_filename="$( basename $last_sourced_script_sub_path )"
-      # When it errors out in sbin/rear last_sourced_script_filename is empty which would result bad looking output
-      # cf. https://github.com/rear/rear/issues/1965#issuecomment-439437868
-      test "$last_sourced_script_filename" || last_sourced_script_filename="$SCRIPT_FILE"
-    } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
-    # Do not log the error message right now but after the currenly last log messages were shown:
+    if test -s "$RUNTIME_LOGFILE" ; then
+        { local last_sourced_script_log_entry=( $( grep -o ' Including .*/[0-9][0-9].*\.sh' $RUNTIME_LOGFILE | tail -n 1 ) )
+          # The last_sourced_script_log_entry contains: Including sub-path/to/script_file_name.sh
+          local last_sourced_script_sub_path="${last_sourced_script_log_entry[1]}"
+          local last_sourced_script_filename="$( basename $last_sourced_script_sub_path )"
+          # When it errors out in sbin/rear last_sourced_script_filename is empty which would result bad looking output
+          # cf. https://github.com/rear/rear/issues/1965#issuecomment-439437868
+          test "$last_sourced_script_filename" || last_sourced_script_filename="$SCRIPT_FILE"
+        } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
+    fi
+    # Do not log the error message right now but after the currently last log messages were shown:
     PrintError "ERROR: $*"
     # Show some additional hopefully meaningful output on the user's terminal
     # (no need to log that again here because it is already in the log file)
     # in particular the normal stdout and stderr messages of the last called programs
     # to make the root cause more obvious to the user without the need to analyze the log file
     # cf. https://github.com/rear/rear/issues/1875#issuecomment-407039065
-    PrintError "Some latest log messages since the last called script $last_sourced_script_filename:"
     # Extract lines starting when the last script was sourced (logged as 'Including sub-path/to/script.sh')
     # but do not use last_sourced_script_sub_path because it contains '/' characters that let sed fail with
     #   sed: -e expression #1, char ...: extra characters after command
@@ -575,12 +628,46 @@ function Error () {
     # Because of the newlines in the error message subsequent lines appear without a leading '+' character
     # so that those debug output lines are indistinguishable from normal stdout/stderr output of programs,
     # cf. https://github.com/rear/rear/pull/1877
-    # Thereafter ('+ [Bug]*Error ' lines were needed before) skip 'set -x' lines (lines that start with a '+' character).
+    # Thereafter ('+ [Bug]*Error ' lines were needed before) skip 'set -x' lines (lines that start with a '+' character)
+    # and skip the initial 'Including sub-path/to/script.sh' line that is always found
+    # to keep only the actual stdout and stderr messages of the last called programs
+    # so we can test if messages were actually found via 'test "string of messages"' for emptiness.
     # Show at most the last 8 lines because too much before the actual error may cause more confusion than help.
     # Add two spaces indentation for better readability what those extracted log file lines are.
     # Some messages could be too long to be usefully shown on the user's terminal so that they are truncated after 200 bytes:
-    { local last_sourced_script_log_messages="$( sed -n -e "/Including .*$last_sourced_script_filename/,/+ [Bug]*Error /p" $RUNTIME_LOGFILE | grep -v '^+' | tail -n 8 | sed -e 's/^/  /' | cut -b-200 )" ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
-    PrintError "$last_sourced_script_log_messages"
+    if test -s "$RUNTIME_LOGFILE" ; then
+        { local last_sourced_script_log_messages="$( sed -n -e "/Including .*$last_sourced_script_filename/,/+ [Bug]*Error /p" $RUNTIME_LOGFILE | egrep -v "^\+|Including .*$last_sourced_script_filename" | tail -n 8 | sed -e 's/^/  /' | cut -b-200 )" ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
+        if test "$last_sourced_script_log_messages" ; then
+            PrintError "Some latest log messages since the last called script $last_sourced_script_filename:"
+            PrintError "$last_sourced_script_log_messages"
+        fi
+    fi
+    # In non-debug modes stdout and stderr are redirected to STDOUT_STDERR_FILE="$TMP_DIR/rear.$WORKFLOW.stdout_stderr" if possible
+    # but in certain cases (e.g. for the 'help' workflow where no $TMP_DIR exists) STDOUT_STDERR_FILE=/dev/null
+    # so we extract some latest messages only if STDOUT_STDERR_FILE is a regular file:
+    if test -f "$STDOUT_STDERR_FILE" ; then
+        # We use the same extraction pipe as above because STDOUT_STDERR_FILE may also contain 'set -x' and things like that
+        # because scripts could use 'set -x' and things like that as needed (e.g. diskrestore.sh runs with 'set -x'):
+        { local last_sourced_script_stdout_stderr_messages="$( sed -n -e "/Including .*$last_sourced_script_filename/,/+ [Bug]*Error /p" $STDOUT_STDERR_FILE | egrep -v "^\+|Including .*$last_sourced_script_filename" | tail -n 8 | sed -e 's/^/  /' | cut -b-200 )" ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
+        if test "$last_sourced_script_stdout_stderr_messages" ; then
+            # When stdout and stderr are redirected to STDOUT_STDERR_FILE messages of the last called programs cannot be in the log
+            # so we use LogPrintError and 'echo "string of messages" >>$RUNTIME_LOGFILE' (the latter avoids the timestamp prefix)
+            # to have the extracted messages stored in the log so that they are later available (in contrast to terminal output).
+            # The full stdout and stderr messages are available in STDOUT_STDERR_FILE:
+            LogPrintError "Some messages from $STDOUT_STDERR_FILE since the last called script $last_sourced_script_filename:"
+            PrintError "$last_sourced_script_stdout_stderr_messages"
+            echo "$last_sourced_script_stdout_stderr_messages" >>"$RUNTIME_LOGFILE"
+        fi
+    fi
+    # Show some generic info about debugging:
+    if test "$DEBUG" ; then
+        # We are in debug mode but not in debugscript mode:
+        test "$DEBUGSCRIPTS" || PrintError "You may use debugscript mode '-D' for full debug messages with 'set -x' output"
+    else
+        # We are not in debug mode:
+        PrintError "Use debug mode '-d' for some debug messages or debugscript mode '-D' for full debug messages with 'set -x' output"
+    fi
+    # Log the error message:
     Log "ERROR: $*"
     LogToSyslog "ERROR: $*"
     # Print stack strace in reverse order to the current STDERR which is (usually) the log file:
@@ -662,7 +749,7 @@ function Error () {
     # except small but crucial differences here which is the reason why that kind of code exists two times.
     # First of all restore the ReaR default bash flags and options of MASTER_PID (i.e. of usr/sbin/rear):
     { apply_bash_flags_and_options_commands "$DEFAULT_BASH_FLAGS_AND_OPTIONS_COMMANDS" ; } 2>/dev/null
-    # Kepp debugscript mode also here if it was used before:
+    # Keep debugscript mode also here if it was used before:
     test "$DEBUGSCRIPTS" && set -$DEBUGSCRIPTS_ARGUMENT
     LogPrint "Error exit of $PROGRAM $WORKFLOW (PID $MASTER_PID) and its descendant processes"
     # Show descendant processes PIDs with their commands in the log
@@ -687,27 +774,87 @@ function Error () {
     fi
 }
 
-# If return code is non-zero, bail out:
-function StopIfError () {
-    if (( $? != 0 )) ; then
-        Error "$@"
-    fi
-}
-
-# Exit if there is a bug in ReaR:
+# Exit if there is a bug inside ReaR:
+# It is a BugError when the cause is in ReaR's code,
+# for example when a ReaR function is called with wrong
+# or missing required parameters and things like that.
 function BugError () {
     { local caller_source="$( CallerSource )" ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
     Error "
 ====================
 BUG in $caller_source:
-'$@'
+'$*'
 --------------------
-Please report this issue at https://github.com/rear/rear/issues
-and include the relevant parts from $RUNTIME_LOGFILE
-preferably with full debug information via 'rear -D $WORKFLOW'
+Please report it at $BUG_REPORT_SITE
+and include all related parts from $RUNTIME_LOGFILE
+preferably the whole debug information via 'rear -D $WORKFLOW'
 ===================="
 }
 
+# Using the ...IfError functions can result unexpected behaviour in certain cases.
+#
+# Using $? in an ...IfError function message like
+#   COMMAND
+#   StopIfError "COMMAND failed with error code $?"
+# lets $? evaluate to an unintended value (usually 0) for example as in
+#   # cat QQQ ; if (( $? != 0 )) ; then echo "ERROR $?" ; fi
+#   cat: QQQ: No such file or directory
+#   ERROR 0
+# In contrast using bash directly as in
+#   # cat QQQ || echo "ERROR $?"
+#   cat: QQQ: No such file or directory
+#   ERROR 1
+# works as expected.
+#
+# The ...IfError functions fail when 'set -e' is set
+# cf. https://github.com/rear/rear/issues/534
+# for example code like
+#   set -e
+#   COMMAND
+#   StopIfError "COMMAND failed"
+# cannot work because 'set -e' exits the script when COMMAND results non-zero exit code
+# so that the subsequent StopIfError is never reached.
+# In contrast using bash directly as in
+#   set -e
+#   COMMAND || Error "COMMAND failed"
+# works as expected.
+#
+# The ...IfError functions fail when $( COMMAND )
+# command substitution is used in the ...IfError functions message
+# cf. https://github.com/rear/rear/issues/1415#issuecomment-315692391
+# for example code like
+#   COMMAND1
+#   StopIfError "... $( COMMAND2 ) ..."
+# cannot work because $? that is tested in the ...IfError functions
+# will be the one from COMMAND2 and not the one from COMMAND1
+# so that StopIfError errors out when COMMAND2 fails but not when COMMAND1 fails.
+# In contrast using bash directly as in
+#   COMMAND1 || Error "... $( COMMAND2 ) ..."
+# works as expected when $? is not used in the Error function message.
+# When $? should be used in the Error function message it must be before $( COMMAND2 ).
+# When $? is before $( COMMAND2 ) it evaluates to the exit code of COMMAND1.
+# When $? is after $( COMMAND2 ) it evaluates to the exit code of COMMAND2.
+# At least with bash-4.4 in openSUSE Leap 15.1 one gets
+#   # cat QQQ || echo "ERROR $? $( grep -Q '/' /etc/fstab ) $?" 
+#   cat: QQQ: No such file or directory
+#   grep: invalid option -- 'Q'
+#   Usage: grep [OPTION]... PATTERN [FILE]...
+#   Try 'grep --help' for more information.
+#   ERROR 1  2
+# when COMMAND1 fails with exit code 1 and COMMAND2 fails with exit code 2
+# versus
+#   # cat QQQ || echo "ERROR $? $( grep -q '/' /etc/fstab ) $?" 
+#   cat: QQQ: No such file or directory
+#   ERROR 1  0
+# when COMMAND1 fails with exit code 1 and COMMAND2 succeeds.
+
+# If return code is non-zero, bail out:
+function StopIfError () {
+    if (( $? != 0 )) ; then
+        Error "$@"
+    fi
+}
+
 # If return code is non-zero, there is a bug in ReaR:
 function BugIfError () {
     if (( $? != 0 )) ; then
@@ -740,6 +887,59 @@ function LogPrintIfError () {
     fi
 }
 
+function cleanup_build_area_and_end_program () {
+    # Cleanup build area
+    local mounted_in_BUILD_DIR
+    Log "Finished $PROGRAM $WORKFLOW in $(( $( date +%s ) - START_SECONDS )) seconds"
+    # is_true is in lib/global-functions.sh which is not yet sourced in case of early Error() in usr/sbin/rear
+    if has_binary is_true && is_true "$KEEP_BUILD_DIR" ; then
+        mounted_in_BUILD_DIR="$( mount | grep "$BUILD_DIR" | sed -e 's/^/  /' )"
+        if test "$mounted_in_BUILD_DIR" ; then
+            LogPrintError "Caution - there is something mounted within the build area"
+            LogPrintError "$mounted_in_BUILD_DIR"
+            LogPrintError "You must manually umount that before you may remove the build area"
+        fi
+        # Show this message also inside the recovery system (e.g. at the end of "rear -D recover")
+        # because there may be a reason why manually removing the build area is wanted
+        # (e.g. some additional manual things need be done before rebooting).
+        # In any case one must be careful if one wants to remove the build area because
+        # e.g. the NFS share with the backup.tar.gz may still be erroneously mounted therein.
+        LogPrint "To remove the build area you may use (with caution): rm -Rf --one-file-system $BUILD_DIR"
+    else
+        Log "Removing build area $BUILD_DIR"
+        # Use '--one-file-system' to be safe against also deleting by accident
+        # all mounted things below mountpoints in TMP_DIR or ROOTFS_DIR
+        # (regardless if mountpoints in TMP_DIR or ROOTFS_DIR may happen):
+        rm -Rf --one-file-system $TMP_DIR || LogPrintError "Failed to 'rm -Rf --one-file-system $TMP_DIR'"
+        rm -Rf --one-file-system $ROOTFS_DIR || LogPrintError "Failed to 'rm -Rf --one-file-system $ROOTFS_DIR'"
+        # Before removing BUILD_DIR check that outputfs is gone (i.e. check that nothing is mounted there):
+        if mountpoint -q "$BUILD_DIR/outputfs" ; then
+            # If still mounted wait a bit (perhaps some ongoing umount needs more time) then try lazy umount:
+            sleep 2
+            # umount_mountpoint_lazy is in lib/global-functions.sh
+            # which is not yet sourced in case of early Error() in usr/sbin/rear
+            has_binary umount_mountpoint_lazy && umount_mountpoint_lazy $BUILD_DIR/outputfs
+        fi
+        # remove_temporary_mountpoint is in lib/global-functions.sh
+        # which is not yet sourced in case of early Error() in usr/sbin/rear
+        if has_binary remove_temporary_mountpoint ; then
+            # It is a bug in ReaR if BUILD_DIR/outputfs was not properly umounted and made empty by the scripts before:
+            remove_temporary_mountpoint "$BUILD_DIR/outputfs" || BugError "Directory $BUILD_DIR/outputfs not empty, cannot remove"
+        fi
+        if ! rmdir $v "$BUILD_DIR" ; then
+            LogPrintError "Could not remove build area $BUILD_DIR (something still exists therein)"
+            mounted_in_BUILD_DIR="$( mount | grep "$BUILD_DIR" | sed -e 's/^/  /' )"
+            if test "$mounted_in_BUILD_DIR" ; then
+                LogPrintError "Something is still mounted within the build area"
+                LogPrintError "$mounted_in_BUILD_DIR"
+                LogPrintError "You must manually umount it, then you could manually remove the build area"
+            fi
+            LogPrintError "To manually remove the build area use (with caution): rm -Rf --one-file-system $BUILD_DIR"
+        fi
+    fi
+    Log "End of program '$PROGRAM' reached"
+}
+
 # UserInput is a general function that is intended for basically any user input.
 #   Output happens via the original STDOUT and STDERR when 'rear' was launched
 #   (which is usually the terminal of the user who launched 'rear') and
@@ -841,9 +1041,8 @@ function LogPrintIfError () {
 #   or the user can interupt any automated response within a relatively short time (minimum is only 1 second).
 function UserInput () {
     # First and foremost log that UserInput was called (but be confidential here):
-    # Have caller_source as an array so that plain $caller_source is only the filename (with path):
-    local caller_source=( $( CallerSource ) )
-    Log "UserInput: called in ${caller_source[@]}"
+    local caller_source="$( CallerSource )"
+    Log "UserInput: called in $caller_source"
     # Set defaults or fallback values:
     # Have a relatively big default timeout of 5 minutes to avoid that the timeout interrupts ongoing user input:
     local timeout=300
@@ -864,9 +1063,9 @@ function UserInput () {
     # Avoid stderr if USER_INPUT_PROMPT is not set or empty:
     test "$USER_INPUT_PROMPT" 2>/dev/null && prompt="$USER_INPUT_PROMPT"
     local input_words_array_name=""
-    local input_max_chars=1000
-    # Avoid stderr if USER_INPUT_MAX_CHARS is not set or empty and ignore wrong USER_INPUT_MAX_CHARS:
-    test "$USER_INPUT_MAX_CHARS" -ge 0 2>/dev/null && input_max_chars=$USER_INPUT_MAX_CHARS
+    local input_max_chars=0
+    # Avoid stderr if USER_INPUT_MAX_CHARS is not set or empty and ignore useless '0' or wrong USER_INPUT_MAX_CHARS:
+    test "$USER_INPUT_MAX_CHARS" -ge 1 2>/dev/null && input_max_chars=$USER_INPUT_MAX_CHARS
     local input_delimiter=""
     local default_input=""
     local user_input_ID=""
@@ -977,7 +1176,7 @@ function UserInput () {
     # The actual work:
     # In debug mode show the user the script that called UserInput and what user_input_ID was specified
     # so that the user can prepare an automated response for that UserInput call (without digging in the code):
-    DebugPrint "UserInput -I $user_input_ID needed in ${caller_source[@]}"
+    DebugPrint "UserInput -I $user_input_ID needed in $caller_source"
     # First of all show the prompt unless an empty prompt was specified (via -p '')
     # so that the prompt can be used as some kind of header line that introduces the user input
     # and separates the following user input from arbitrary other output lines before:
@@ -1067,8 +1266,13 @@ function UserInput () {
         # Regarding how to get all array elements when the array name is in a variable, see
         # https://unix.stackexchange.com/questions/60584/how-to-use-a-variable-as-part-of-an-array-name
         # Assume input_words_array_name="myarr" then input_words_array_name_dereferenced="myarr[*]"
-        # and "${!input_words_array_name_dereferenced}" becomes "${myarr[*]}":
-        local input_words_array_name_dereferenced="$input_words_array_name[*]"
+        # and "${!input_words_array_name_dereferenced}" becomes "${myarr[*]}"
+        # Avoid ShellCheck false error indication for code like
+        #   string_appended="$string[*]"
+        #                    ^-- SC1087: Use braces when expanding arrays, e.g. ${array[idx]}
+        # by appending '[*]' to a string variable in a separated command:
+        local input_words_array_name_dereferenced="$input_words_array_name"
+        input_words_array_name_dereferenced+='[*]'
         input_string="${!input_words_array_name_dereferenced}"
     fi
     # When there is no user input or when the user input is only spaces use the "best" fallback or default that exists.
