@@ -56,6 +56,10 @@ function find_syslinux_modules_dir {
             # not default location? try to find it
             # file=/usr/lib/syslinux/modules/efi32/menu.c32
             # f23: file=/usr/share/syslinux/menu.c32
+            # Because 'find /usr' may take very long time on some systems
+            # cf. https://github.com/rear/rear/issues/2792
+            # tell the user in debug mode what is going on
+            DebugPrint "Searching whole /usr for SYSLINUX modules directory (you may specify SYSLINUX_MODULES_DIR)"
             file=$( find /usr -name "$1" 2>/dev/null | tail -1 )
             syslinux_modules_dir=$( dirname "$file" )        # /usr/lib/syslinux/modules/efi32
             syslinux_modules_dir=${syslinux_modules_dir%/*}  # /usr/lib/syslinux/modules
@@ -174,19 +178,13 @@ function set_syslinux_features {
     FEATURE_SYSLINUX_IS_SET=1
 }
 
-
-
 # Create a suitable syslinux configuration based on capabilities
-# the mandatory first argument is the full path to an existing directory where required
-# binaries will be copied to
+# the mandatory first argument is the full path to an existing directory where required binaries will be copied to
 # the optional second argment is the target flavour and defaults to isolinux
 function make_syslinux_config {
-    [[ -d "$1" ]]
-    BugIfError "Required argument for BOOT_DIR is missing"
-    [[ -d "$SYSLINUX_DIR" ]]
-    BugIfError "Required environment SYSLINUX_DIR ($SYSLINUX_DIR) is not set or not a directory"
-    [[ "$FEATURE_SYSLINUX_IS_SET" ]]
-    BugIfError "You must call set_syslinux_features before"
+    test -d "$1" || BugError "make_syslinux_config: required first argument for BOOT_DIR missing"
+    test -d "$SYSLINUX_DIR" || BugError "make_syslinux_config: required environment SYSLINUX_DIR '$SYSLINUX_DIR' not set or not a directory"
+    test "$FEATURE_SYSLINUX_IS_SET" || BugError "make_syslinux_config: set_syslinux_features must be called before"
 
     local BOOT_DIR="$1" ; shift
     local flavour="${1:-isolinux}" ; shift
@@ -199,17 +197,55 @@ function make_syslinux_config {
         SYSLINUX_DIR="$syslinux_modules_dir"
     fi
 
-    # Enable serial console, unless explicitly disabled (only last entry is used :-/)
-    if [[ "$USE_SERIAL_CONSOLE" =~ ^[yY1] ]]; then
-        for devnode in $(ls /dev/ttyS[0-9]* | sort); do
-            speed=$(stty -F $devnode 2>/dev/null | awk '/^speed / { print $2 }')
-            if [ "$speed" ]; then
-                echo "serial ${devnode##/dev/ttyS} $speed"
+    # Enable serial console:
+    # For the SERIAL directive to work properly, it must be the first directive in the configuration file,
+    # see "SERIAL port" at https://wiki.syslinux.org/wiki/index.php?title=SYSLINUX
+    # It may be useful to reduce it to exact one device since the last 'serial' line wins in SYSLINUX.
+    if is_true "$USE_SERIAL_CONSOLE" ; then
+        # When the user has specified SERIAL_CONSOLE_DEVICE_SYSLINUX use only that (no automatisms):
+        if test "$SERIAL_CONSOLE_DEVICE_SYSLINUX" ; then
+            # SERIAL_CONSOLE_DEVICE_SYSLINUX can be a character device node like "/dev/ttyS0"
+            # or a whole SYSLINUX 'serial' directive like "serial 1 9600" for /dev/ttyS1 at 9600 baud.
+            if test -c "$SERIAL_CONSOLE_DEVICE_SYSLINUX" ; then
+                # The port value for the SYSLINUX 'serial' directive
+                # is the trailing digits of the serial device node
+                # cf. the code of get_partition_number() in lib/layout-functions.sh
+                port=$( echo "$SERIAL_CONSOLE_DEVICE_SYSLINUX" | grep -o -E '[0-9]+$' )
+                # E.g. for /dev/ttyS12 the unit would be 12 but
+                # https://wiki.syslinux.org/wiki/index.php?title=SYSLINUX
+                # reads in the section "SERIAL port [baudrate [flowcontrol]]"
+                # "port values from 0 to 3 mean the first four serial ports detected by the BIOS"
+                # which indicates port values should be less than 4 so we tell the user about it
+                # but we do not error out because the user may have tested that it does work for him:
+                test $port -lt 4 || LogPrintError "SERIAL_CONSOLE_DEVICE_SYSLINUX '$SERIAL_CONSOLE_DEVICE_SYSLINUX' may not work (only /dev/ttyS0 up to /dev/ttyS3 should work)"
+                if speed=$( get_serial_device_speed $SERIAL_CONSOLE_DEVICE_SYSLINUX ) ; then
+                    echo "serial $port $speed"
+                else
+                    echo "serial $port"
+                fi
+            else
+                # When SERIAL_CONSOLE_DEVICE_SYSLINUX is a whole SYSLINUX 'serial' directive use it as specified:
+                echo "$SERIAL_CONSOLE_DEVICE_SYSLINUX"
             fi
-        done
+        else
+            for devnode in $( get_serial_console_devices ) ; do
+                # Add SYSLINUX serial console config for real serial devices:
+                if speed=$( get_serial_device_speed $devnode ) ; then
+                    # The port value for the SYSLINUX 'serial' directive
+                    # is the trailing digits of the serial device node
+                    # cf. the code of get_partition_number() in lib/layout-functions.sh
+                    port=$( echo "$devnode" | grep -o -E '[0-9]+$' )
+                    test $port -lt 4 || LogPrintError "$devnode may not work as serial console for SYSLINUX (only /dev/ttyS0 up to /dev/ttyS3 should work)"
+                    echo "serial $port $speed"
+                    # Use the first one and skip the rest to avoid that the last 'serial' line wins in SYSLINUX:
+                    break
+                fi
+            done
+        fi
     fi
-
-    # if we have the menu.c32 available we use it. if not we make sure that there will be no menu lines in the result
+    
+    # if we have the menu.c32 available we use it
+    # if not we make sure that there will be no menu lines in the result
     # so that we don't confuse older syslinux
     if [[ -r "$SYSLINUX_DIR/menu.c32" ]] ; then
         cp $v "$SYSLINUX_DIR/menu.c32" "$BOOT_DIR/menu.c32" >&2
@@ -465,7 +501,7 @@ function make_syslinux_config {
 # Create configuration file for elilo
 function create_ebiso_elilo_conf {
 cat << EOF
-timeout = 5
+timeout = 300
 default = "Relax-and-Recover (no Secure Boot)"
 
 image = kernel
@@ -488,61 +524,226 @@ function get_root_disk_UUID {
     # because /.snapshots is one word for grep -w and then those two matches result
     # two same UUIDs (with newline) that end up (with newline) in the boot menuentry
     # cf. https://github.com/rear/rear/issues/1871
-    echo $(mount | grep ' on / ' | awk '{print $1}' | xargs blkid -s UUID -o value)
+    # Run it in a subshell so that 'set -o pipefail' does not affect the current shell and
+    # it can run in a subshell because the caller of this function only needs its stdout:
+    ( set -o pipefail ; mount | grep ' on / ' | awk '{print $1}' | xargs blkid -s UUID -o value || Error "Failed to get root disk UUID" )
 }
 
-# Create configuration grub
+# Output GRUB2 configuration on stdout:
+# $1 is the kernel file with appropriate path for GRUB2 to load the kernel from within GRUB2's root filesystem
+# $2 is the initrd file with appropriate path for GRUB2 to load the initrd from within GRUB2's root filesystem
+# e.g. when a separated boot partition that contains kernel and initrd is currently mounted at /boot_mountpoint
+# so that kernel and initrd are /boot_mountpoint/path/to/kernel and /boot_mountpoint/path/to/initrd
+# and that boot partition gets set as root device name for GRUB2's
+# then $1 would have to be /path/to/kernel and $2 would have to be /path/to/initrd
+# $3 is an appropriate GRUB2 command to set its root device (usually via GRUB2's 'root' environment variable)
+# e.g. when the filesystem that contains kernel and initrd has the filesystem label REARBOOT
+# then $3 could be something like 'search --no-floppy --set root --label REARBOOT'
 function create_grub2_cfg {
-    root_uuid=$(get_root_disk_UUID)
+    local grub2_kernel="$1"
+    test "$grub2_kernel" || BugError "create_grub2_cfg function called without grub2_kernel argument"
+    DebugPrint "Configuring GRUB2 kernel $grub2_kernel"
+    local grub2_initrd="$2"
+    test "$grub2_initrd" || BugError "create_grub2_cfg function called without grub2_initrd argument"
+    DebugPrint "Configuring GRUB2 initrd $grub2_initrd"
+    local grub2_search_root_command="$3"
+    if ! test "$grub2_search_root_command" ; then
+        test "$grub2_set_root" && grub2_search_root_command="set root=$grub2_set_root"
+    fi
+    if ! test "$grub2_search_root_command" ; then
+        test "$GRUB2_SEARCH_ROOT_COMMAND" && grub2_search_root_command="$GRUB2_SEARCH_ROOT_COMMAND"
+    fi
+    test "$grub2_search_root_command" || grub2_search_root_command="search --no-floppy --set=root --file /boot/efiboot.img"
+    DebugPrint "Configuring GRUB2 root device as '$grub2_search_root_command'"
 
-    local esp_kernel="$1"
-    local esp_initrd="$2"
+    local grub2_default_menu_entry="$GRUB2_DEFAULT_BOOT"
+    test "$grub2_default_menu_entry" || grub2_default_menu_entry="chainloader"
 
-    test $esp_kernel || esp_kernel="/isolinux/kernel"
-    test $esp_initrd || esp_initrd="/isolinux/$REAR_INITRD_FILENAME"
+    local grub2_timeout="$GRUB2_TIMEOUT"
+    test "$grub2_timeout" || grub2_timeout=300
 
-    cat << EOF
-set default="$GRUB2_DEFAULT_BOOT"
+    local root_uuid=$( get_root_disk_UUID )
+    test -b /dev/disk/by-uuid/$root_uuid || Error "root_uuid device '/dev/disk/by-uuid/$root_uuid' is no block device"
 
-insmod all_video
+    # Local helper functions for the create_grub2_cfg function:
 
-set gfxpayload=keep
-insmod gzio
-insmod part_gpt
-insmod ext2
+    function create_grub2_serial_entry {
+        # Enable serial console:
+        # It may be useful to reduce it to exact one device since the last 'serial' line wins in GRUB.
+        if is_true "$USE_SERIAL_CONSOLE" ; then
+            # When the user has specified SERIAL_CONSOLE_DEVICE_GRUB use only that (no automatisms):
+            if test "$SERIAL_CONSOLE_DEVICE_GRUB" ; then
+                # SERIAL_CONSOLE_DEVICE_GRUB can be a character device node like "/dev/ttyS0"
+                # or a whole GRUB 'serial' command like "serial --unit=1 --speed=9600"
+                if test -c "$SERIAL_CONSOLE_DEVICE_GRUB" ; then
+                    # The unit value for the GRUB 'serial' command
+                    # is the trailing digits of the serial device node
+                    # cf. the code of get_partition_number() in lib/layout-functions.sh
+                    unit=$( echo "$SERIAL_CONSOLE_DEVICE_GRUB" | grep -o -E '[0-9]+$' )
+                    # E.g. for /dev/ttyS12 the unit would be 12 but
+                    # https://www.gnu.org/software/grub/manual/grub/grub.html#serial
+                    # reads "unit is a number in the range 0-3" so we tell the user about it
+                    # but we do not error out because the user may have tested that it does work for him:
+                    test $unit -lt 4 || LogPrintError "SERIAL_CONSOLE_DEVICE_GRUB '$SERIAL_CONSOLE_DEVICE_GRUB' may not work (only /dev/ttyS0 up to /dev/ttyS3 should work)"
+                    if speed=$( get_serial_device_speed $SERIAL_CONSOLE_DEVICE_GRUB ) ; then
+                        echo "serial --unit=$unit --speed=$speed"
+                    else
+                        echo "serial --unit=$unit"
+                    fi
+                else
+                    # When SERIAL_CONSOLE_DEVICE_GRUB is a whole GRUB 'serial' command use it as specified:
+                    echo "$SERIAL_CONSOLE_DEVICE_GRUB"
+                fi
+            else
+                for devnode in $( get_serial_console_devices ) ; do
+                    # Add GRUB serial console config for real serial devices:
+                    if speed=$( get_serial_device_speed $devnode ) ; then
+                        # The unit value for the GRUB 'serial' command
+                        # is the trailing digits of the serial device node
+                        # cf. the code of get_partition_number() in lib/layout-functions.sh
+                        unit=$( echo "$devnode" | grep -o -E '[0-9]+$' )
+                        test $unit -lt 4 || LogPrintError "$devnode may not work as serial console for GRUB (only /dev/ttyS0 up to /dev/ttyS3 should work)"
+                        echo "serial --unit=$unit --speed=$speed"
+                        # Use the first one and skip the rest to avoid that the last 'serial' line wins in GRUB:
+                        break
+                    fi
+                done
+            fi
+            if is_true $GRUB_FORCE_SERIAL ; then
+                echo "terminal_input serial"
+                echo "terminal_output serial"
+            fi
+        fi
+    }
 
-set timeout=5
-
-search --no-floppy --file /boot/efiboot.img --set
-$grub2_set_usb_root
-
-menuentry "Relax-and-Recover (no Secure Boot)"  --class gnu-linux --class gnu --class os {
-     echo 'Loading kernel ...'
-     linux $esp_kernel root=UUID=$root_uuid $KERNEL_CMDLINE
-     echo 'Loading initial ramdisk ...'
-     initrd $esp_initrd
+    function create_grub2_rear_boot_entry {
+        # "ReaR (BIOS or UEFI without Secure Boot)"
+        # is correct in case you don't use EFI (i.e. for BIOS)
+        # and should work for EFI with secure boot disabled.
+        # "ReaR (UEFI and Secure Boot)"
+        # only works with EFI (and you may need secure boot enabled).
+        if is_true $USING_UEFI_BOOTLOADER ; then
+            cat << EOF
+menuentry "Relax-and-Recover (BIOS or UEFI without Secure Boot)" --id=rear {
+    insmod gzio
+    insmod xzio
+    echo 'Loading kernel $grub2_kernel ...'
+    linux $grub2_kernel root=UUID=$root_uuid $KERNEL_CMDLINE
+    echo 'Loading initial ramdisk $grub2_initrd ...'
+    initrd $grub2_initrd
 }
 
-menuentry "Relax-and-Recover (Secure Boot)"  --class gnu-linux --class gnu --class os {
-     echo 'Loading kernel ...'
-     linuxefi $esp_kernel root=UUID=$root_uuid $KERNEL_CMDLINE
-     echo 'Loading initial ramdisk ...'
-     initrdefi $esp_initrd
+menuentry "Relax-and-Recover (UEFI and Secure Boot)" --id=rear_secure_boot {
+    insmod gzio
+    insmod xzio
+    echo 'Loading kernel $grub2_kernel ...'
+    linuxefi $grub2_kernel root=UUID=$root_uuid $KERNEL_CMDLINE
+    echo 'Loading initial ramdisk $grub2_initrd ...'
+    initrdefi $grub2_initrd
 }
+EOF
+        else
+            cat << EOF
+menuentry "Relax-and-Recover (BIOS or UEFI in legacy BIOS mode)" --id=rear {
+    insmod gzio
+    insmod xzio
+    echo 'Loading kernel $grub2_kernel ...'
+    linux $grub2_kernel root=UUID=$root_uuid $KERNEL_CMDLINE
+    echo 'Loading initial ramdisk $grub2_initrd ...'
+    initrd $grub2_initrd
+}
+EOF
+        fi
+    # End of function create_grub2_rear_boot_entry
+    }
 
-menuentry "Boot original system" {
+    function create_grub2_boot_next_entry {
+        if is_true $USING_UEFI_BOOTLOADER ; then
+            # Search next EFI and chainload it.
+            # FIXME: The "GNU GRUB Manual" https://www.gnu.org/software/grub/manual/grub/grub.html
+            # does not mention a GRUB environment variable 'esp' so "--set=esp" could be wrong.
+            # Perhaps it is a typo and "--set=root $esp_disk_uuid" is meant?
+            # If actually "--set=esp $esp_disk_uuid" is meant provide a comment
+            # that explains what that "--set=esp" is and where it is documented.
+            cat << EOF
+menuentry "Boot next EFI" --id=chainloader {
+    insmod chain
     search --fs-uuid --no-floppy --set=esp $esp_disk_uuid
     chainloader (\$esp)$esp_relative_bootloader
 }
-
-menuentry "Reboot" {
-     reboot
-}
-
-menuentry "Exit to EFI Shell" {
-     exit
+EOF
+        else
+            # Try booting from second disk hd1 that is usually the original system disk
+            # because the first disk hd0 in the USB disk wherefrom currently is booted:
+            cat << EOF
+menuentry "Boot from second disk hd1 (usually the original system disk)" --id=chainloader {
+    insmod chain
+    set root=(hd1)
+    chainloader +1
 }
 EOF
+        fi
+    # End of function create_grub2_boot_next_entry
+    }
+
+    function create_grub2_reboot_entry {
+        cat << EOF
+menuentry "Reboot" --id=reboot {
+    reboot
+}
+EOF
+    }
+
+    function create_grub2_exit_entry {
+        if is_true $USING_UEFI_BOOTLOADER ; then 
+            cat << EOF
+menuentry "Exit to EFI shell" --id=exit {
+    exit
+}
+EOF
+        else
+            cat << EOF
+menuentry "Exit (possibly continue bootchain)" --id=exit {
+    exit
+}
+EOF
+        fi
+    }
+
+    # End of local helper functions for the create_grub2_cfg function
+
+    # The actual work starts here.
+    # Create and output GRUB2 configuration.
+    # Sleep 3 seconds before the GRUB2 menu replaces what there is on the screen
+    # so that the user has a chance to see possible (error) messages on the screen.
+    cat << EOF
+$grub2_search_root_command
+insmod all_video
+set gfxpayload=keep
+insmod part_gpt
+insmod part_msdos
+insmod ext2
+$( create_grub2_serial_entry )
+set timeout="$grub2_timeout"
+set default="$grub2_default_menu_entry"
+set fallback="chainloader"
+echo 'Switching to GRUB2 boot menu...'
+sleep --verbose --interruptible 3
+$( create_grub2_rear_boot_entry )
+$( create_grub2_boot_next_entry )
+$( create_grub2_reboot_entry )
+$( create_grub2_exit_entry )
+EOF
+
+    # Local functions must be 'unset' because bash does not support 'local function ...'
+    # cf. https://unix.stackexchange.com/questions/104755/how-can-i-create-a-local-function-in-my-bashrc
+    unset -f create_grub2_serial_entry
+    unset -f create_grub2_rear_boot_entry
+    unset -f create_grub2_boot_next_entry
+    unset -f create_grub2_reboot_entry
+    unset -f create_grub2_exit_entry
+
+# End of function create_grub2_cfg
 }
 
 function make_pxelinux_config {
@@ -588,6 +789,34 @@ function make_pxelinux_config {
     echo "    append initrd=$PXE_INITRD root=/dev/ram0 vga=normal rw $KERNEL_CMDLINE $PXE_RECOVER_MODE"
     echo "say ----------------------------------------------------------"
 
+    # start with optional rear http entry if specified
+    if [[ ! -z $PXE_HTTP_URL ]] ; then    
+        case "$PXE_RECOVER_MODE" in
+        "automatic")
+            echo "say rear-automatic-http - Recover $HOSTNAME (HTTP) with auto-recover kernel option"
+            echo "label rear-automatic-http"
+            echo "MENU label ^Automatic Recover $HOSTNAME (HTTP)"
+            ;;
+        "unattended")
+            echo "say rear-unattended-http - Recover $HOSTNAME (HTTP) with unattended kernel option"
+            echo "label rear-unattended-http"
+            echo "MENU label ^Unattended Recover $HOSTNAME (HTTP)"
+            ;;
+        *)
+            echo "say rear-http - Recover $HOSTNAME (HTTP)"
+            echo "label rear-http"
+            echo "MENU label ^Recover $HOSTNAME (HTTP)"
+            ;;
+        esac
+        echo "TEXT HELP"
+        echo "Rescue image kernel $KERNEL_VERSION ${IPADDR:+on $IPADDR} $(date -R)"
+        echo "${BACKUP:+BACKUP=$BACKUP} ${OUTPUT:+OUTPUT=$OUTPUT} ${BACKUP_URL:+BACKUP_URL=$BACKUP_URL}"
+        echo "ENDTEXT"
+        echo "    kernel $PXE_HTTP_URL/$PXE_KERNEL"
+        echo "    append initrd=$PXE_HTTP_URL/$PXE_INITRD root=/dev/ram0 vga=normal rw $KERNEL_CMDLINE $PXE_RECOVER_MODE"
+        echo "say ----------------------------------------------------------"
+    fi
+
     # start the the other entries like local,...
     echo "say local - Boot from next boot device"
     echo "label local"
