@@ -237,12 +237,27 @@ function percent_decode() {
 ######
 # URL is the most common form of URI
 # see https://en.wikipedia.org/wiki/Uniform_Resource_Identifier
-# where a generic URI is of the form
-# scheme:[//[user:password@]host[:port]][/]path[?query][#fragment]
+# where a generic URI is usually of the form
+# scheme://[[user:password@]host[:port]]/path[?query][#fragment]
 # e.g. for BACKUP_URL=sshfs://user@host/G/rear/
 # url_scheme = 'sshfs' , url_host = 'user@host' , url_hostname = 'host' , url_username = 'user' , url_path = '/G/rear/'
 # e.g. for BACKUP_URL=usb:///dev/sdb1
 # url_scheme = 'usb' , url_host = '' , url_hostname = '' , url_username = '' , url_path = '/dev/sdb1'
+# TODO: the url_* functions do not support the minimal scheme:path case of an URL
+# for example
+#   # url='mailto:John.Doe@example.com'
+#   # url_scheme "$url"
+#   rsync
+#   # url_host "$url"
+#   mailto:John.Doe@example.com
+#   # url_hostname "$url"
+#   example.com
+#   # url_username "$url"
+#   mailto
+#   # url_password "$url"
+#   John.Doe
+#   # url_path "$url"
+#   /mailto:John.Doe@example.com
 # FIXME: the ulr_* functions are not safe against special characters
 # for example they break when the password contains spaces
 # but on the other hand permitted characters for values in a URI
@@ -259,7 +274,7 @@ function url_scheme() {
     # the scheme is the leading part up to '://'
     local scheme=${url%%://*}
     # rsync scheme does not have to start with rsync:// it can also be scp style
-    # see the comments in usr/share/rear/prep/RSYNC/default/100_check_rsync.sh
+    # see the comments in usr/share/rear/lib/rsync-functions.sh
     echo $scheme | grep -q ":" && echo rsync || echo $scheme
 }
 
@@ -342,7 +357,60 @@ function url_path() {
     echo /${url_without_scheme#*/}
 }
 
-backup_path() {
+### Returns true if one can upload files to the URL
+function scheme_accepts_files() {
+    # Be safe against 'set -eu' which would exit 'rear' with "bash: $1: unbound variable"
+    # when scheme_accepts_files is called without an argument
+    # by bash parameter expansion with using an empty default value if $1 is unset or null.
+    # Bash parameter expansion with assigning a default value ${1:=} does not work
+    # (then it would still exit with "bash: $1: cannot assign in this way")
+    # but using a default value is practicable here because $1 is used only once
+    # cf. https://github.com/rear/rear/pull/2675#discussion_r705018956
+    local scheme=${1:-}
+    # Return false if scheme is empty or blank (e.g. when OUTPUT_URL is unset or empty or blank)
+    # cf. https://github.com/rear/rear/issues/2676
+    # and https://github.com/rear/rear/issues/2667#issuecomment-914447326
+    # also return false if scheme is more than one word (so no quoted "$scheme" here)
+    # cf. https://github.com/rear/rear/pull/2675#discussion_r704401462
+    test $scheme || return 1
+    case $scheme in
+        (null|tape|obdr)
+            # tapes do not support uploading arbitrary files, one has to handle them
+            # as special case (usually passing the tape device as argument to tar)
+            # null means do not upload anything anywhere, leave the files under /var/lib/rear/output
+            return 1
+            ;;
+        (*)
+            # most URL schemes support uploading files
+            return 0
+            ;;
+    esac
+}
+
+### Returns true if URLs with the given scheme corresponds to a path inside
+### a mountable fileystem and one can put files directly into it.
+### The actual path will be returned by backup_path() / output_path().
+### If returns false, using backup_path() / output_path() has no sense
+### and one must use a scheme-specific method (like lftp or writing them to a tape)
+### to upload files to the destination instead of just "cp" or other direct filesystem access.
+### Returning true does not imply that the URL is currently mounted at a filesystem and usable,
+### only that it can be mounted (use mount_url() first)
+function scheme_supports_filesystem() {
+    # Be safe against 'set -eu' exit if scheme_supports_filesystem is called without argument
+    local scheme=${1:-}
+    # Return false if scheme is empty or blank or more than one word, cf. scheme_accepts_files() above
+    test $scheme || return 1
+    case $scheme in
+        (null|tape|obdr|rsync|fish|ftp|ftps|hftp|http|https|sftp)
+            return 1
+            ;;
+        (*)
+            return 0
+            ;;
+    esac
+}
+
+function backup_path() {
     local scheme=$1
     local path=$2
     case $scheme in
@@ -368,13 +436,21 @@ backup_path() {
     echo "$path"
 }
 
-output_path() {
+function output_path() {
     local scheme=$1
     local path=$2
+
+    # Abort for unmountable schemes ("tape-like" or "ftp-like" schemes).
+    # Returning an empty string for them is not satisfactory: it could lead to caller putting its files
+    # under / instead of the intended location if the result is not checked for emptiness.
+    # Returning ${BUILD_DIR}/outputfs/${OUTPUT_PREFIX} for unmountable URLs is also not satisfactory:
+    # caller could put its files there expecting them to be safely at their destination,
+    # but if the directory is not a mountpoint, they would get silently lost.
+    # The caller needs to check the URL/scheme using scheme_supports_filesystem()
+    # before calling this function.
+    scheme_supports_filesystem $scheme || BugError "output_path() called with scheme $scheme that does not support filesystem access"
+
     case $scheme in
-       (null|tape)  # no path for tape required
-           path=""
-           ;;
        (file)  # type file needs a local path (must be mounted by user)
            path="$path/${OUTPUT_PREFIX}"
            ;;
@@ -387,23 +463,39 @@ output_path() {
 
 
 ### Mount URL $1 at mountpoint $2[, with options $3]
-mount_url() {
+function mount_url() {
     local url=$1
     local mountpoint=$2
     local defaultoptions="rw,noatime"
     local options=${3:-"$defaultoptions"}
+    local scheme
+
+    scheme=$( url_scheme $url )
+
+    # The cases where we return 0 are those that do not need umount and also do not need ExitTask handling.
+    # They thus need to be kept in sync with umount_url() so that RemoveExitTasks is used
+    # iff AddExitTask was used in mount_url().
+
+    if ! scheme_supports_filesystem $scheme ; then
+        ### Stuff like null|tape|rsync|fish|ftp|ftps|hftp|http|https|sftp
+        ### Don't need to umount anything for these.
+        ### file: supports filesystem access, but is not mounted and unmounted,
+        ### so it has to be handled specially below.
+        ### Similarly for iso: which gets mounted and unmounted only during recovery.
+        return 0
+    fi
 
     ### Generate a mount command
     local mount_cmd
-    case $(url_scheme $url) in
-        (null|tape|file|rsync|fish|ftp|ftps|hftp|http|https|sftp)
-            ### Don't need to mount anything for these
+    case $scheme in
+        (file)
+            ### Don't need to mount anything for file:, it is already mounted by user
             return 0
             ;;
         (iso)
-            # Check that there is a symbolic link /dev/disk/by-label/RELAXRECOVER
-            # that points to a block device that uses the filesystem label RELAXRECOVER.
-            # RELAXRECOVER is the default value of the ISO_VOLID config variable.
+            # Check that there is a symbolic link /dev/disk/by-label/REAR-ISO
+            # that points to a block device that uses the filesystem label REAR-ISO.
+            # REAR-ISO is the default value of the ISO_VOLID config variable.
             # If no such symbolic link exists create one because it is needed
             # during "rear recover" when the ISO image contains the backup,
             # see https://github.com/rear/rear/issues/1893
@@ -412,39 +504,39 @@ mount_url() {
             # so that there is nothing to do here unless during "rear recover":
             test "recover" = "$WORKFLOW" || return 0
             # Try to find a block device that uses the filesystem label ISO_VOLID.
-            # Usually "blkid -L RELAXRECOVER" results '/dev/sr0' or '/dev/sr1'
+            # Usually "blkid -L REAR-ISO" results '/dev/sr0' or '/dev/sr1'
             # cf. https://github.com/rear/rear/issues/1893#issuecomment-411034001
             # but "blkid -L" is not supported on SLES10 (blkid is too old there)
-            # so that the traditional form "blkid -l -o device -t LABEL=RELAXRECOVER"
+            # so that the traditional form "blkid -l -o device -t LABEL=REAR-ISO"
             # is used which also works and is described in "man blkid" on SLES15:
-            local relaxrecover_block_device="$( blkid -l -o device -t LABEL="$ISO_VOLID" )"
+            local iso_block_device="$( blkid -l -o device -t LABEL="$ISO_VOLID" )"
             # Try to get where the symbolic link /dev/disk/by-label/ISO_VOLID points to.
             # "readlink -e symlink" outputs nothing when the symlink or its target does not exist:
-            local relaxrecover_symlink_name="/dev/disk/by-label/$ISO_VOLID"
-            local relaxrecover_symlink_target="$( readlink $verbose -e "$relaxrecover_symlink_name" )"
-            # Everything is o.k. when relaxrecover_block_device and relaxrecover_symlink_target are non-empty
-            # and when the relaxrecover_symlink_target value is the relaxrecover_block_device value.
+            local iso_symlink_name="/dev/disk/by-label/$ISO_VOLID"
+            local iso_symlink_target="$( readlink $verbose -e "$iso_symlink_name" )"
+            # Everything is o.k. when iso_block_device and iso_symlink_target are non-empty
+            # and when the iso_symlink_target value is the iso_block_device value.
             # Usually the right symbolic link /dev/disk/by-label/ISO_VOLID is set up automatically by udev.
-            if ! test "$relaxrecover_block_device" -a "$relaxrecover_symlink_target" -a "$relaxrecover_symlink_target" = "$relaxrecover_block_device" ; then
+            if ! test "$iso_block_device" -a "$iso_symlink_target" -a "$iso_symlink_target" = "$iso_block_device" ; then
                 # If not everything is o.k. first try fix things automatically:
-                Log "Symlink '$relaxrecover_symlink_name' does not exist or does not point to a block device with '$ISO_VOLID' filesystem label"
+                Log "Symlink '$iso_symlink_name' does not exist or does not point to a block device with '$ISO_VOLID' filesystem label"
                 # One of the things that could be not o.k. is that there is no /dev/disk/by-label/ directory.
                 # Usually udev would automatically create it but sometimes that does not work,
                 # cf. https://github.com/rear/rear/issues/1891#issuecomment-411027324
                 # so that we create a /dev/disk/by-label/ directory if it is not there:
                 mkdir $verbose -p /dev/disk/by-label
                 # Try to let the symbolic link point to the block device that uses the filesystem label ISO_VOLID:
-                if test -b "$relaxrecover_block_device" ; then
-                    Log "Making symlink '$relaxrecover_symlink_name' point to '$relaxrecover_block_device' because it has filesystem label '$ISO_VOLID'"
+                if test -b "$iso_block_device" ; then
+                    Log "Making symlink '$iso_symlink_name' point to '$iso_block_device' because it has filesystem label '$ISO_VOLID'"
                     # Below there is a test that /dev/disk/by-label/ISO_VOLID exists which should detect when this 'ln' command failed:
-                    ln $verbose -sf "$relaxrecover_block_device" "$relaxrecover_symlink_name"
+                    ln $verbose -sf "$iso_block_device" "$iso_symlink_name"
                 else
                     # We found no block device that uses the filesystem label ISO_VOLID:
                     Log "No block device with ISO filesystem label '$ISO_VOLID' found (by the blkid command)"
                     # At this point things look not good so that now we need to tell the user about what is wrong:
-                    LogPrintError "A symlink '$relaxrecover_symlink_name' is required that points to the device with the ReaR ISO image"
+                    LogPrintError "A symlink '$iso_symlink_name' is required that points to the device with the ReaR ISO image"
                     rear_workflow="rear $WORKFLOW"
-                    rear_shell_history="$( echo -e "ln -vsf /dev/cdrom $relaxrecover_symlink_name\nls -l $relaxrecover_symlink_name" )"
+                    rear_shell_history="$( echo -e "ln -vsf /dev/cdrom $iso_symlink_name\nls -l $iso_symlink_name" )"
                     unset choices
                     choices[0]="/dev/cdrom is where the ISO is attached to"
                     choices[1]="/dev/sr0 is where the ISO is attached to"
@@ -452,35 +544,35 @@ mount_url() {
                     choices[3]="Use Relax-and-Recover shell and return back to here"
                     choices[4]="Continue '$rear_workflow'"
                     choices[5]="Abort '$rear_workflow'"
-                    prompt="Create symlink '$relaxrecover_symlink_name' that points to the ReaR ISO image device"
+                    prompt="Create symlink '$iso_symlink_name' that points to the ReaR ISO image device"
                     choice=""
                     wilful_input=""
                     symlink_target=""
-                    # When USER_INPUT_RELAXRECOVER_SYMLINK_TARGET has any 'true' value be liberal in what you accept and
-                    # assume choices[0] 'Let /dev/disk/by-label/RELAXRECOVER point to /dev/cdrom' was actually meant:
-                    is_true "$USER_INPUT_RELAXRECOVER_SYMLINK_TARGET" && USER_INPUT_RELAXRECOVER_SYMLINK_TARGET="${choices[0]}"
+                    # When USER_INPUT_ISO_SYMLINK_TARGET has any 'true' value be liberal in what you accept and
+                    # assume choices[0] 'Let /dev/disk/by-label/REAR-ISO point to /dev/cdrom' was actually meant:
+                    is_true "$USER_INPUT_ISO_SYMLINK_TARGET" && USER_INPUT_ISO_SYMLINK_TARGET="${choices[0]}"
                     while true ; do
-                        choice="$( UserInput -I RELAXRECOVER_SYMLINK_TARGET -p "$prompt" -D "${choices[0]}" "${choices[@]}" )" && wilful_input="yes" || wilful_input="no"
+                        choice="$( UserInput -I ISO_SYMLINK_TARGET -p "$prompt" -D "${choices[0]}" "${choices[@]}" )" && wilful_input="yes" || wilful_input="no"
                         case "$choice" in
                             (${choices[0]})
                                 symlink_target="/dev/cdrom"
                                 is_true "$wilful_input" && LogPrint "User confirmed symlink target $symlink_target" || LogPrint "Using symlink target $symlink_target by default"
                                 # Below there is a test that /dev/disk/by-label/ISO_VOLID exists which should detect when this 'ln' command failed:
-                                ln $verbose -sf $symlink_target "$relaxrecover_symlink_name"
+                                ln $verbose -sf $symlink_target "$iso_symlink_name"
                                 break
                                 ;;
                             (${choices[1]})
                                 symlink_target="/dev/sr0"
                                 LogPrint "Using symlink target $symlink_target"
                                 # Below there is a test that /dev/disk/by-label/ISO_VOLID exists which should detect when this 'ln' command failed:
-                                ln $verbose -sf $symlink_target "$relaxrecover_symlink_name"
+                                ln $verbose -sf $symlink_target "$iso_symlink_name"
                                 break
                                 ;;
                             (${choices[2]})
                                 symlink_target="/dev/sr1"
                                 LogPrint "Using symlink target $symlink_target"
                                 # Below there is a test that /dev/disk/by-label/ISO_VOLID exists which should detect when this 'ln' command failed:
-                                ln $verbose -sf $symlink_target "$relaxrecover_symlink_name"
+                                ln $verbose -sf $symlink_target "$iso_symlink_name"
                                 break
                                 ;;
                             (${choices[3]})
@@ -500,8 +592,8 @@ mount_url() {
                 fi
             fi
             # Check if /dev/disk/by-label/$ISO_VOLID exists (as symbolic link or in any other form), if yes assume things are right:
-            test -e "$relaxrecover_symlink_name" || Error "Cannot mount ISO because there is no '$relaxrecover_symlink_name'"
-            mount_cmd="mount $relaxrecover_symlink_name $mountpoint"
+            test -e "$iso_symlink_name" || Error "Cannot mount ISO because there is no '$iso_symlink_name'"
+            mount_cmd="mount $iso_symlink_name $mountpoint"
             ;;
         (var)
             ### The mount command is given by variable in the url host
@@ -558,22 +650,47 @@ mount_url() {
             ;;
     esac
 
+    # create mount point
+    mkdir -p $v "$mountpoint" || Error "Could not mkdir '$mountpoint'"
+    AddExitTask "remove_temporary_mountpoint '$mountpoint'"
+
     Log "Mounting with '$mount_cmd'"
     # eval is required when mount_cmd contains single quoted stuff (e.g. see the above mount_cmd for curlftpfs)
     eval $mount_cmd || Error "Mount command '$mount_cmd' failed."
 
-    AddExitTask "umount -f $v '$mountpoint' >&2"
+    AddExitTask "perform_umount_url '$url' '$mountpoint' lazy"
     return 0
 }
 
-### Unmount url $1 at mountpoint $2
-umount_url() {
+function remove_temporary_mountpoint() {
+    if test -d "$1" ; then
+        rmdir $v "$1"
+    fi
+}
+
+### Unmount url $1 at mountpoint $2, perform mountpoint cleanup and exit task + error handling
+function umount_url() {
     local url=$1
     local mountpoint=$2
+    local scheme
 
-    case $(url_scheme $url) in
-        (null|tape|file|rsync|fish|ftp|ftps|hftp|http|https|sftp)
-            ### Don't need to umount anything for these
+    scheme=$( url_scheme $url )
+
+    # The cases where we return 0 are those that do not need umount and also do not need ExitTask handling.
+    # They thus need to be kept in sync with mount_url() so that RemoveExitTasks is used
+    # iff AddExitTask was used in mount_url().
+
+    if ! scheme_supports_filesystem $scheme ; then
+        ### Stuff like null|tape|rsync|fish|ftp|ftps|hftp|http|https|sftp
+        ### Don't need to umount anything for these.
+        ### file: supports filesystem access, but is not mounted and unmounted,
+        ### so it has to be handled specially below.
+        ### Similarly for iso: which gets mounted and unmounted only during recovery.
+        return 0
+    fi
+
+    case $scheme in
+        (file)
             return 0
             ;;
         (iso)
@@ -581,42 +698,106 @@ umount_url() {
                 return 0
             fi
             ;;
-	    (sshfs)
-	        umount_cmd="fusermount -u $mountpoint"
-	    ;;
-	    (davfs)
-	        umount_cmd="umount $mountpoint"
-            # Wait for 3 sek. then remove the cache-dir /var/cache/davfs
-            sleep 30
-            # ToDo: put in here the cache-dir from /etc/davfs2/davfs.conf
-            # and delete only the just used cache
-            #rm -rf /var/cache/davfs2/*<mountpoint-hash>*
-            rm -rf /var/cache/davfs2/*outputfs*
-
-	    ;;
-        (var)
-            local var=$(url_host $url)
-            umount_cmd="${!var} $mountpoint"
-
-            Log "Unmounting with '$umount_cmd'"
-            $umount_cmd
-            StopIfError "Unmounting failed."
-
-            RemoveExitTask "umount -f $v '$mountpoint' >&2"
-            return 0
-            ;;
+        (*)
+            # Schemes that actually need nontrivial umount are handled below.
+            # We do not handle them in the default branch because in the case of iso:
+            # it depends on the current workflow whether umount is needed or not.
+            :
     esac
 
-    umount_mountpoint $mountpoint
-    StopIfError "Unmounting '$mountpoint' failed."
+    # umount_url() is a wrapper that takes care of exit tasks and error handling and mountpoint cleanup.
+    # Therefore it also determines if exit task and mountpoint handling is required and returns early if not.
+    # The actual umount job is performed inside perform_umount_url().
+    # We do not request lazy umount here because we want umount errors to be reliably reported.
+    perform_umount_url $url $mountpoint || Error "Unmounting '$mountpoint' failed."
 
-    RemoveExitTask "umount -f $v '$mountpoint' >&2"
+    RemoveExitTask "perform_umount_url '$url' '$mountpoint' lazy"
+
+    remove_temporary_mountpoint "$mountpoint" && RemoveExitTask "remove_temporary_mountpoint '$mountpoint'"
     return 0
 }
 
-### Unmount mountpoint $1
-umount_mountpoint() {
+### Unmount url $1 at mountpoint $2 [ lazily if $3 is set to 'lazy' and normal unmount fails ]
+function perform_umount_url() {
+    local url=$1
+    local mountpoint=$2
+    local lazy=${3:-}
+
+    if test $lazy ; then
+        if test $lazy != "lazy" ; then
+            BugError "lazy = $lazy, but it must have the value of 'lazy' or empty"
+        fi
+    fi
+
+    case $(url_scheme $url) in
+        (sshfs)
+            # does ftpfs need this special case as well?
+            fusermount -u ${lazy:+'-z'} $mountpoint
+            ;;
+        (davfs)
+            umount_davfs $mountpoint $lazy
+            ;;
+        (var)
+            local var
+            var=$(url_host $url)
+            Log "Unmounting with '${!var} $mountpoint'"
+            # lazy unmount not supported with custom umount command
+            ${!var} $mountpoint
+            ;;
+        (*)
+            # usual umount command
+            umount_mountpoint $mountpoint $lazy
+    esac
+    # The switch above must be the last statement in this function and the umount commands must be
+    # the last commands (or part of) in each branch. This ensures proper exit code propagation
+    # to the caller even when set -e is used.
+}
+
+### Helper which unmounts davfs mountpoint $1 and cleans up the cache,
+### performing lazy unmount if $2 = 'lazy' and normal unmount fails.
+function umount_davfs() {
     local mountpoint=$1
+    local lazy="${2:-}"
+
+    if test $lazy ; then
+        if test $lazy != "lazy" ; then
+            BugError "lazy = $lazy, but it must have the value of 'lazy' or empty"
+        fi
+    fi
+
+    if umount_mountpoint $mountpoint ; then
+        # Wait for 3 sek. then remove the cache-dir /var/cache/davfs
+        sleep 30
+        # TODO: put in here the cache-dir from /etc/davfs2/davfs.conf
+        # and delete only the just used cache
+        #rm -rf /var/cache/davfs2/*<mountpoint-hash>*
+        rm -rf /var/cache/davfs2/*outputfs*
+    else
+        local retval=$?
+
+        if test $lazy ; then
+            # try again to unmount lazily and this time do not delete the cache, it is still in use.
+            LogPrintError "davfs cache /var/cache/davfs2/*outputfs* needs to be cleaned up manually after the lazy unmount finishes"
+            umount_mountpoint_lazy $mountpoint
+        else
+            # propagate errors from umount
+            return $retval
+        fi
+    fi
+}
+
+### Unmount mountpoint $1 [ lazily if $2 = 'lazy' ]
+### Default implementation for filesystems that don't need anything fancy
+### For special umount commands use perform_umount_url()
+function umount_mountpoint() {
+    local mountpoint=$1
+    local lazy=${2:-}
+
+    if test $lazy ; then
+        if test $lazy != "lazy" ; then
+            BugError "lazy = $lazy, but it must have the value of 'lazy' or empty"
+        fi
+    fi
 
     ### First, try a normal unmount,
     Log "Unmounting '$mountpoint'"
@@ -636,7 +817,21 @@ umount_mountpoint() {
     fi
 
     Log "Unmounting '$mountpoint' failed."
-    return 1
+
+    if test $lazy ; then
+        umount_mountpoint_lazy $mountpoint
+    else
+        return 1
+    fi
+}
+
+### Unmount mountpoint $1 lazily
+### Preferably use "umount_mountpoint $mountpoint lazy", which attempts non-lazy unmount first.
+function umount_mountpoint_lazy() {
+    local mountpoint=$1
+
+    LogPrint "Directory $mountpoint still mounted - trying lazy umount"
+    umount $v -f -l $mountpoint >&2
 }
 
 # Change $1 to user input or leave default value on empty input
