@@ -1,36 +1,30 @@
 # Utility functions for the system layout processing.
 
-# TODO: Why not using ISO 8601 date 'date +%F' ?
-# Be exact to one second to ensure saved original files
-# of directly subsequent 'rear WORKFLOW' are saved with different filenames
-# that could help to avoid confusion when debugging issues later:
-DATE=$( date +%Y%m%d%H%M%S )
-
 # Each file will be only saved once by save_original_file()
 # and all subsequent save_original_file() for the same file do nothing
 # because each saved file is remembered in the SAVED_ORIGINAL_FILES array:
 SAVED_ORIGINAL_FILES=()
 SAVED_ORIGINAL_FILE_SUFFIX="orig"
 
-# Save the original content of the file $1 to $1.$DATE.$2.orig
-# or to $1.$DATE.$WORKFLOW.$MASTER_PID.orig when no $2 is specified:
+# Save the original content of the file $1 to $1.$START_DATE_TIME_NUMBER.$2.orig
+# or to $1.$START_DATE_TIME_NUMBER.$WORKFLOW.$MASTER_PID.orig when no $2 is specified:
 save_original_file() {
     local filename="$1"
     test -r "$filename" || return 1
     IsInArray "$filename" "${SAVED_ORIGINAL_FILES[@]}" && return 0
     local extension="$2"
     test "$extension" || extension=$WORKFLOW.$MASTER_PID
-    local saved_original_file="$filename.$DATE.$extension.$SAVED_ORIGINAL_FILE_SUFFIX"
+    local saved_original_file="$filename.$START_DATE_TIME_NUMBER.$extension.$SAVED_ORIGINAL_FILE_SUFFIX"
     cp -ar $filename $saved_original_file && SAVED_ORIGINAL_FILES+=( "$filename" )
 }
 
 # Restore the saved original content of the original file named $1
-# that was saved as $1.$DATE.$2.orig or $1.$DATE.$WORKFLOW.$MASTER_PID.orig
+# that was saved as $1.$START_DATE_TIME_NUMBER.$2.orig or $1.$START_DATE_TIME_NUMBER.$WORKFLOW.$MASTER_PID.orig
 restore_original_file() {
     local filename="$1"
     local extension="$2"
     test "$extension" || extension=$WORKFLOW.$MASTER_PID
-    local saved_original_file="$filename.$DATE.$extension.$SAVED_ORIGINAL_FILE_SUFFIX"
+    local saved_original_file="$filename.$START_DATE_TIME_NUMBER.$extension.$SAVED_ORIGINAL_FILE_SUFFIX"
     test -r "$saved_original_file" || return 1
     cp -ar $saved_original_file $filename
 }
@@ -168,13 +162,13 @@ generate_layout_dependencies() {
                 [ -z "$pool" ] || add_dependency "$dm_prefix-$dm_lvol" "$dm_prefix-$dm_pool"
                 add_component "$dm_prefix-$dm_lvol" "lvmvol"
                 ;;
-            raid)
+            raidarray)
                 name=$(echo "$remainder" | cut -d " " -f "1")
-                disks=( $(echo "$remainder" | sed -r "s/.*devices=([^ ]+).*/\1/" | tr ',' ' ') )
-                for disk in "${disks[@]}" ; do
+                disks=$(echo "$remainder" | sed -r "s/.*devices=([^ ]+).*/\1/" | tr ',' ' ')
+                for disk in $disks ; do
                     add_dependency "$name" "$disk"
                 done
-                add_component "$name" "raid"
+                add_component "$name" "raidarray"
                 ;;
             fs|btrfsmountedsubvol)
                 dev=$(echo "$remainder" | cut -d " " -f "1")
@@ -217,9 +211,7 @@ generate_layout_dependencies() {
             multipath)
                 name=$(echo "$remainder" | cut -d " " -f "1")
                 disks=$(echo "$remainder" | cut -d " " -f "4" | tr "," " ")
-
                 add_component "$name" "multipath"
-
                 for disk in $disks ; do
                     add_dependency "$name" "$disk"
                 done
@@ -227,7 +219,9 @@ generate_layout_dependencies() {
             opaldisk)
                 dev=$(echo "$remainder" | cut -d " " -f "1")
                 add_component "opaldisk:$dev" "opaldisk"
-                add_dependency "$dev" "opaldisk:$dev"
+                for disk in $(opal_device_disks "$dev"); do
+                    add_dependency "$disk" "opaldisk:$dev"
+                done
                 ;;
         esac
     done < $LAYOUT_FILE
@@ -247,16 +241,34 @@ add_component() {
     echo "todo $1 $2" >> $LAYOUT_TODO
 }
 
-# Mark device $1 as done.
+# The distinction in the mark_as_done and mark_tree_as_done functions what messages should appear
+# - only in the log file in debug '-d' mode via 'Debug'
+# - in the log file and on the user's terminal in debug '-d' mode via 'DebugPrint'
+# matches the same kind of distinction in the disable_component_... functions
+# in layout/save/default/330_remove_exclusions.sh
+# but no LogPrint is used in the lower-level mark_as_done and mark_tree_as_done functions.
+
+# Mark component $1 as done.
 mark_as_done() {
-    Debug "Marking $1 as done."
+    # The trailing blank in "... $1 " is crucial to not match wrong components
+    # for example the component "... /dev/sda1" must not match accidentally
+    # other components like "... /dev/sda12" in var/lib/rear/layout/disktodo.conf
+    if grep -q "done $1 " $LAYOUT_TODO ; then
+        DebugPrint "Component '$1' is marked as 'done $1' in $LAYOUT_TODO"
+        return 0
+    fi
+    if ! grep -q "todo $1 " $LAYOUT_TODO ; then
+        Debug "Cannot mark component '$1' as done because there is no 'todo $1 ' in $LAYOUT_TODO"
+        return 1
+    fi
+    DebugPrint "Marking component '$1' as done in $LAYOUT_TODO"
     sed -i "s;todo\ $1\ ;done\ $1\ ;" $LAYOUT_TODO
 }
 
-# Mark all devices that depend on $1 as done.
+# Mark all components that depend on component $1 as done.
 mark_tree_as_done() {
-    for component in $(get_child_components "$1") ; do
-        Debug "Marking $component as done (dependent on $1)"
+    for component in $( get_child_components "$1" ) ; do
+        DebugPrint "Dependant component $component is a child of component $1"
         mark_as_done "$component"
     done
 }
@@ -302,12 +314,20 @@ get_child_components() {
     done
 }
 
-# Return all ancestors of component $1 [ of type $2 ]
+# Return all ancestors of component $1 [ of type $2 [ skipping types $3 during resolution ] ]
 get_parent_components() {
-    declare -a ancestors devlist
-    declare current child parent
+    declare -a ancestors devlist ignoretypes
+    declare current child parent parenttype
 
     devlist=( "$1" )
+    if [[ "$3" ]] ; then
+        # third argument should, if present, be a space-separated list
+        # of types to ignore when walking up the dependency tree.
+        # Convert it to array
+        ignoretypes=( $3 )
+    else
+        ignoretypes=()
+    fi
     while (( ${#devlist[@]} )) ; do
         current=${devlist[0]}
 
@@ -318,6 +338,13 @@ get_parent_components() {
                 if IsInArray "$parent" "${ancestors[@]}" ; then
                     continue
                 fi
+                ### ...test if parent is of a correct type if requested...
+                if [[ ${#ignoretypes[@]} -gt 0 ]] ; then
+                    parenttype=$(get_component_type "$parent")
+                    if IsInArray "$parenttype" "${ignoretypes[@]}" ; then
+                        continue
+                    fi
+                fi
                 ### ...and add them to the list
                 devlist+=( "$parent" )
                 ancestors+=( "$parent" )
@@ -345,22 +372,24 @@ get_parent_components() {
 }
 
 # find_devices <other>
+# ${2+"$2"} in the following functions ensures that $2 gets passed down quoted if present
+# and ignored if not present
 # Find the disk device(s) component $1 resides on.
 find_disk() {
-    get_parent_components "$1" "disk"
+    get_parent_components "$1" "disk" ${2+"$2"}
 }
 
 find_multipath() {
-    get_parent_components "$1" "multipath"
+    get_parent_components "$1" "multipath" ${2+"$2"}
 }
 
 find_disk_and_multipath() {
-    find_disk "$1"
-    is_true "$AUTOEXCLUDE_MULTIPATH" || find_multipath "$1"
+    find_disk "$1" ${2+"$2"}
+    is_true "$AUTOEXCLUDE_MULTIPATH" || find_multipath "$1" ${2+"$2"}
 }
 
 find_partition() {
-    get_parent_components "$1" "part"
+    get_parent_components "$1" "part" ${2+"$2"}
 }
 
 # The get_partition_number function
@@ -413,6 +442,54 @@ get_partition_number() {
     echo $partition_number
 }
 
+# Extract the underlying device name from the full partition device name.
+# Underlying device may be a disk, a multipath device or other devices that can be partitioned.
+# Should we use the information in $LAYOUT_DEPS, like get_parent_component does,
+# instead of string munging?
+function get_device_from_partition() {
+    local partition_block_device
+    local device
+    local partition_number
+
+    partition_block_device=$1
+    test -b "$partition_block_device" || BugError "get_device_from_partition called with '$partition_block_device' that is no block device"
+    partition_number=${2-$(get_partition_number $partition_block_device )}
+    # /dev/sda or /dev/mapper/vol34_part or /dev/mapper/mpath99p or /dev/mmcblk0p
+    device=${partition_block_device%$partition_number}
+
+    # Strip trailing partition remainders like '_part' or '-part' or 'p'
+    # if we have 'mapper' in disk device name:
+    if [[ ${partition_block_device/mapper//} != $partition_block_device ]] ; then
+        # we only expect mpath_partX or mpathpX or mpath-partX
+        case $device in
+            (*p)     device=${device%p} ;;
+            (*-part) device=${device%-part} ;;
+            (*_part) device=${device%_part} ;;
+            (*)      Log "Unsupported kpartx partition delimiter for $partition_block_device"
+        esac
+    fi
+
+    # For eMMC devices the trailing 'p' in the $device value
+    # (as in /dev/mmcblk0p that is derived from /dev/mmcblk0p1)
+    # needs to be stripped (to get /dev/mmcblk0), otherwise the
+    # efibootmgr call fails because of a wrong disk device name.
+    # See also https://github.com/rear/rear/issues/2103
+    if [[ $device = *'/mmcblk'+([0-9])p ]] ; then
+        device=${device%p}
+    fi
+
+    # For NVMe devices the trailing 'p' in the $device value
+    # (as in /dev/nvme0n1p that is derived from /dev/nvme0n1p1)
+    # needs to be stripped (to get /dev/nvme0n1), otherwise the
+    # efibootmgr call fails because of a wrong disk device name.
+    # See also https://github.com/rear/rear/issues/1564
+    if [[ $device = *'/nvme'+([0-9])n+([0-9])p ]] ; then
+        device=${device%p}
+    fi
+
+    test -b "$device" && echo $device
+}
+
 # Returns partition start block or 'unknown'
 # sda/sda1 or
 # dm-XX
@@ -486,7 +563,7 @@ version_newer() {
 
 # Function to get version from tool.
 get_version() {
-  TERM=dumb $@ 2>&1 | sed -rn 's/^[^0-9\.]*([0-9]+\.[-0-9a-z\.]+).*$/\1/p' | head -1
+  TERM=dumb "$@" 2>&1 | sed -rn 's/^[^0-9\.]*([0-9]+\.[-0-9a-z\.]+).*$/\1/p' | head -1
 }
 
 # Translate a device name to a sysfs name.
@@ -696,17 +773,38 @@ is_disk_a_pv() {
     return 0
 }
 
-function is_multipath_path {
-    # Return 'false' if there is no device as argument:
-    test "$1" || return 1
+function is_multipath_used {
     # Return 'false' if there is no multipath command:
     type multipath &>/dev/null || return 1
-    # Return 'false' if multipath is not used, see https://github.com/rear/rear/issues/2298
+    # 'multipath -l' is the only simple and reliably working commad
+    # to find out in general whether or not multipath is used at all.
+    # But 'multipath -l' scans all devices and the time it takes is proportional
+    # to their number so that time would become rather long (seconds up to minutes)
+    # if 'multipath -l' was called for each one of hundreds or thousands of devices.
+    # So we call 'multipath -l' only once and remember the result
+    # in a global variable and then only use that global variable
+    # so we can call is_multipath_used very many times as often as needed.
+    is_true $MULTIPATH_IS_USED && return 0
+    is_false $MULTIPATH_IS_USED && return 1
+    # When MULTIPATH_IS_USED has neither a true nor false value set it and return accordingly.
     # Because "multipath -l" always returns zero exit code we check if it has real output via grep -q '[[:alnum:]]'
     # so that no "multipath -l" output could clutter the log (the "multipath -l" output is irrelevant here)
     # in contrast to e.g. test "$( multipath -l )" that would falsely succeed with blank output
     # and the output would appear in the log in 'set -x' debugscript mode:
-    multipath -l | grep -q '[[:alnum:]]' || return 1
+    if multipath -l | grep -q '[[:alnum:]]' ; then
+        MULTIPATH_IS_USED='yes'
+        return 0
+    else
+        MULTIPATH_IS_USED='no'
+        return 1
+    fi
+}
+
+function is_multipath_path {
+    # Return 'false' if there is no device as argument:
+    test "$1" || return 1
+    # Return 'false' if multipath is not used, see https://github.com/rear/rear/issues/2298
+    is_multipath_used || return 1
     # Check if a block device should be a path in a multipath device:
     multipath -c /dev/$1 &>/dev/null
 }
@@ -1282,7 +1380,7 @@ delete_dummy_partitions_and_resize_real_ones() {
 
     # Delete dummy partitions
     local -i num
-    for num in ${dummy_partitions_to_delete[@]} ; do
+    for num in "${dummy_partitions_to_delete[@]}" ; do
         LogPrint "Disk '$current_disk': deleting dummy partition number $num"
         parted -s -m $current_disk rm $num
     done
