@@ -1,5 +1,8 @@
 # Create a suitable syslinux configuration based on capabilities
 
+# Nothing to do here when GRUB2 is specified to be used as USB bootloader:
+test "$USB_BOOTLOADER" = "grub" && return
+
 function get_usb_syslinux_version {
     for file in $BUILD_DIR/outputfs/{boot/syslinux,}/{ld,ext}linux.sys; do
         if [[ -s "$file" ]];  then
@@ -54,7 +57,7 @@ function syslinux_has {
 # FIXME: Syslinux older than 3.62 do have menu.c32 but not submenu support
 #        We simplify by disabling MENU support for everything older than 3.62
 function syslinux_write {
-    if [[ "$@" ]]; then
+    if [[ "$*" ]]; then
         echo "$@" | syslinux_write
     elif [[ "$FEATURE_SYSLINUX_SUBMENU" ]]; then
         cat >&4
@@ -97,8 +100,7 @@ esac
 
 USB_REAR_DIR="$BUILD_DIR/outputfs/$USB_PREFIX"
 if [ ! -d "$USB_REAR_DIR" ]; then
-    mkdir -p $v "$USB_REAR_DIR" >/dev/null
-    StopIfError "Could not create USB ReaR dir [$USB_REAR_DIR] !"
+    mkdir -p $v "$USB_REAR_DIR" >/dev/null || Error "Could not create USB ReaR dir [$USB_REAR_DIR] !"
 fi
 
 # We generate a single syslinux.cfg for the current system
@@ -141,6 +143,13 @@ EOF
 
 # Clean up older images of a given system, but keep USB_RETAIN_BACKUP_NR
 # entries for backup and rescue when backup on USB works in default mode.
+# It keeps by default the two (by default USB_RETAIN_BACKUP_NR=2)
+# topmost directories of the "ls -dt $BUILD_DIR/outputfs/rear/$HOSTNAME/*"
+# output ("ls -t" sorts by time, newest first)
+# regardless whether or not a new backup or a new rescue system was created
+# so when there is no new backup or no new rescue system
+# it keeps the existing directories as is,
+# cf. https://github.com/rear/rear/pull/2794#issuecomment-1106286485
 # When USB_SUFFIX is set the compliance mode is used where
 # backup on USB works in compliance with backup on NFS which means
 # a fixed backup directory and no automated removal of backups or other stuff
@@ -251,8 +260,7 @@ EOF
 } 4>"$BUILD_DIR/outputfs/rear/syslinux.cfg"
 
 if [ ! -d "$BUILD_DIR/outputfs/$SYSLINUX_PREFIX" ]; then
-    mkdir -p $v "$BUILD_DIR/outputfs/$SYSLINUX_PREFIX" >/dev/null
-    StopIfError "Could not create USB syslinux dir [$BUILD_DIR/outputfs/$SYSLINUX_PREFIX] !"
+    mkdir -p $v "$BUILD_DIR/outputfs/$SYSLINUX_PREFIX" >/dev/null || Error "Could not create USB syslinux dir [$BUILD_DIR/outputfs/$SYSLINUX_PREFIX] !"
 fi
 
 echo "$VERSION_INFO" >$BUILD_DIR/outputfs/$SYSLINUX_PREFIX/message
@@ -261,14 +269,51 @@ echo "$VERSION_INFO" >$BUILD_DIR/outputfs/$SYSLINUX_PREFIX/message
 # default functionality
 Log "Creating $SYSLINUX_PREFIX/extlinux.conf"
 {
-    # Enable serial console, unless explicitly disabled
-    if [[ "$USE_SERIAL_CONSOLE" =~ ^[yY1] ]]; then
-        for devnode in $(ls /dev/ttyS[0-9]* | sort); do
-            speed=$(stty -F $devnode 2>/dev/null | awk '/^speed / { print $2 }')
-            if [ "$speed" ]; then
-                syslinux_write "serial ${devnode##/dev/ttyS} $speed"
+    # Enable serial console:
+    # For the SERIAL directive to work properly, it must be the first directive in the configuration file,
+    # see "SERIAL port" at https://wiki.syslinux.org/wiki/index.php?title=SYSLINUX
+    # It may be useful to reduce it to exact one device since the last 'serial' line wins in SYSLINUX.
+    if is_true "$USE_SERIAL_CONSOLE" ; then
+        # When the user has specified SERIAL_CONSOLE_DEVICE_SYSLINUX use only that (no automatisms):
+        if test "$SERIAL_CONSOLE_DEVICE_SYSLINUX" ; then
+            # SERIAL_CONSOLE_DEVICE_SYSLINUX can be a character device node like "/dev/ttyS0"
+            # or a whole SYSLINUX 'serial' directive like "serial 1 9600" for /dev/ttyS1 at 9600 baud.
+            if test -c "$SERIAL_CONSOLE_DEVICE_SYSLINUX" ; then
+                # The port value for the SYSLINUX 'serial' directive
+                # is the trailing digits of the serial device node
+                # cf. the code of get_partition_number() in lib/layout-functions.sh
+                port=$( echo "$SERIAL_CONSOLE_DEVICE_SYSLINUX" | grep -o -E '[0-9]+$' )
+                # E.g. for /dev/ttyS12 the unit would be 12 but
+                # https://wiki.syslinux.org/wiki/index.php?title=SYSLINUX
+                # reads in the section "SERIAL port [baudrate [flowcontrol]]"
+                # "port values from 0 to 3 mean the first four serial ports detected by the BIOS"
+                # which indicates port values should be less than 4 so we tell the user about it
+                # but we do not error out because the user may have tested that it does work for him:
+                test $port -lt 4 || LogPrintError "SERIAL_CONSOLE_DEVICE_SYSLINUX '$SERIAL_CONSOLE_DEVICE_SYSLINUX' may not work (only /dev/ttyS0 up to /dev/ttyS3 should work)"
+                if speed=$( get_serial_device_speed $SERIAL_CONSOLE_DEVICE_SYSLINUX ) ; then
+                    syslinux_write "serial $port $speed"
+                else
+                    syslinux_write "serial $port"
+                fi
+            else
+                # When SERIAL_CONSOLE_DEVICE_SYSLINUX is a whole SYSLINUX 'serial' directive use it as specified:
+                syslinux_write "$SERIAL_CONSOLE_DEVICE_SYSLINUX"
             fi
-        done
+        else
+            for devnode in $( get_serial_console_devices ) ; do
+                # Add SYSLINUX serial console config for real serial devices:
+                if speed=$( get_serial_device_speed $devnode ) ; then
+                    # The port value for the SYSLINUX 'serial' directive
+                    # is the trailing digits of the serial device node
+                    # cf. the code of get_partition_number() in lib/layout-functions.sh
+                    port=$( echo "$devnode" | grep -o -E '[0-9]+$' )
+                    test $port -lt 4 || LogPrintError "$devnode may not work as serial console for SYSLINUX (only /dev/ttyS0 up to /dev/ttyS3 should work)"
+                    syslinux_write "serial $port $speed"
+                    # Use the first one and skip the rest to avoid that the last 'serial' line wins in SYSLINUX:
+                    break
+                fi
+            done
+        fi
     fi
 
     syslinux_write "display message"
