@@ -3,22 +3,20 @@
 [ -z "${KEEP_OLD_OUTPUT_COPY}" ] && return
 
 # do not do this for tapes and special attention for file:///path
-url="$( echo $stage | tr '[:lower:]' '[:upper:]')_URL"
-local scheme=$(url_scheme ${!url})
-local path=$(url_path ${!url})
-local opath=$(output_path $scheme $path)
+local scheme=$( url_scheme $OUTPUT_URL )
+local path=$( url_path $OUTPUT_URL )
 
-# if $opath is empty return silently (e.g. scheme tape)
-[ -z "$opath" ] && return 0
+# if filesystem access to url is unsupported return silently (e.g. scheme tape)
+scheme_supports_filesystem $scheme || return 0
+
+local opath=$( output_path $scheme $path )
 
 # an old lockfile from a previous run not cleaned up by output is possible
 [[ -f ${opath}/.lockfile ]] && rm -f ${opath}/.lockfile >&2
 
 if test -d "${opath}" ; then
-    rm -rf $v "${opath}.old" >&2
-    StopIfError "Could not remove '${opath}.old'"
+    rm -rf $v "${opath}.old" || Error "Could not remove '${opath}.old'"
     # below statement was 'cp -af' instead of 'mv -f' (see issue #192)
-    mv -f $v "${opath}" "${opath}.old" >&2
-    StopIfError "Could not move '${opath}'"
+    mv -f $v "${opath}" "${opath}.old" || Error "Could not move '${opath}'"
 fi
 # the ${BUILD_DIR}/outputfs/${OUTPUT_PREFIX} will be created by output/default/200_make_prefix_dir.sh
