@@ -1,10 +1,11 @@
 # remove the lockfile
 local scheme=$(url_scheme $OUTPUT_URL)
 local path=$(url_path $OUTPUT_URL)
-local opath=$(output_path $scheme $path)
 
-# if $opath is empty return silently (e.g. scheme tape)
-[ -z "$opath" ] && return 0
+# if filesystem access to url is unsupported return silently (e.g. scheme tape)
+scheme_supports_filesystem $scheme || return 0
+
+local opath=$( output_path $scheme $path )
 
 # when OUTPUT_URL=BACKUP_URL we keep the lockfile to avoid double moves of the directory
 [[ "$OUTPUT_URL" != "$BACKUP_URL" ]] && rm -f $v "${opath}/.lockfile" >&2
