@@ -1,18 +1,21 @@
-# For serial support we need to include the agetty binary, but Debian distro's
-# use getty instead of agetty.
 
-# Enable serial console if possible, when not specified
-if [[ -z "$USE_SERIAL_CONSOLE" ]]; then
-    for devnode in $(ls /dev/ttyS[0-9]* | sort); do
-        if stty -F $devnode >/dev/null 2>&1; then
-            USE_SERIAL_CONSOLE=y
+# If possible auto-enable serial console when not specified:
+if [[ -z "$USE_SERIAL_CONSOLE" ]] ; then
+    local devnode speed=""
+    for devnode in $( get_serial_console_devices ) ; do
+        # Enable serial console when there is at least one real serial device:
+        if speed=$( get_serial_device_speed $devnode ) ; then
+            USE_SERIAL_CONSOLE="yes"
+            break
         fi
     done
 fi
 
 # Always include getty or agetty as we don't know in advance whether they are needed
 # (the user may boot the recovery system with manually specified kernel options
-# to get serial console support in his recovery system):
+# to get serial console support in his recovery system).
+# For serial support we need to include the agetty binary,
+# but Debian distro's use getty instead of agetty:
 local getty_binary=""
 if has_binary getty ; then
     # Debian, Ubuntu,...
