@@ -3,7 +3,4 @@
 grep -q blocks /proc/mdstat 2>/dev/null || return 0
 
 Log "Software RAID detected. Including mdadm tools."
-
-PROGS=( "${PROGS[@]}"
-mdadm
-)
+PROGS+=( mdadm )
