@@ -1,3 +1,4 @@
+#
 # 400_restore_backup.sh
 #
 
@@ -22,18 +23,6 @@ local restore_input_basename=""
 
 mkdir -p $BUILD_DIR/outputfs/$NETFS_PREFIX
 
-# Disable BACKUP_PROG_DECRYPT_OPTIONS by replacing the default with 'cat' when encryption is disabled
-# (by default encryption is disabled but the default BACKUP_PROG_DECRYPT_OPTIONS is not 'cat'):
-if is_true "$BACKUP_PROG_CRYPT_ENABLED" ; then
-    # Backup archive decryption is only supported with 'tar':
-    test "tar" = "$BACKUP_PROG" || Error "Backup archive decryption is only supported with BACKUP_PROG=tar"
-    LogPrint "Decrypting backup archive with key defined in variable \$BACKUP_PROG_CRYPT_KEY"
-else
-    Log "Decrypting backup archive is disabled"
-    BACKUP_PROG_DECRYPT_OPTIONS="cat"
-    BACKUP_PROG_CRYPT_KEY=""
-fi
-
 # The RESTORE_ARCHIVES array contains the restore input files.
 # If it is not set, RESTORE_ARCHIVES is only one element which is the backup archive:
 test "$RESTORE_ARCHIVES" || RESTORE_ARCHIVES=( "$backuparchive" )
@@ -51,9 +40,9 @@ if test -f $TMP_DIR/backup.splitted ; then
         Print ""
         while read backup_splitted_line ; do
             # The lines in backup.splitted are like
-            #   backup.tar.gz.00 878706688 RELAXRECOVER
-            #   backup.tar.gz.01 878706688 RELAXRECOVER_01
-            #   backup.tar.gz.02 758343480 RELAXRECOVER_02
+            #   backup.tar.gz.00 878706688 REAR-ISO
+            #   backup.tar.gz.01 878706688 REAR-ISO_01
+            #   backup.tar.gz.02 758343480 REAR-ISO_02
             # The first word is backup file name, the second a size, the last one is the label/vol_name:
             backup_file_name=${backup_splitted_line%% *}
             vol_name=${backup_splitted_line##* }
@@ -99,7 +88,9 @@ if test -f $TMP_DIR/backup.splitted ; then
                 ProgressInfo ""
                 LogPrint "Processing $backup_file_name ..."
                 # The actual feeder program:
-                dd if="$backup_file_path" of="$FIFO"
+                # Let 'dd' read and write up to 1M=1024*1024 bytes at a time to speed up things
+                # cf. https://github.com/rear/rear/issues/2369 and https://github.com/rear/rear/issues/2458
+                dd if="$backup_file_path" of="$FIFO" bs=1M
             else
                 StopIfError "$backup_file_name could not be found on the $vol_name medium!"
             fi
@@ -137,17 +128,37 @@ for restore_input in "${RESTORE_ARCHIVES[@]}" ; do
     # output of a possibly simultaneously running process that likes to append to the log file
     # (e.g. when background processes run that also uses the log file for logging)
     # cf. https://github.com/rear/rear/issues/885#issuecomment-310308763
+    # Do not show the BACKUP_PROG_CRYPT_KEY value in a log file
+    # where BACKUP_PROG_CRYPT_KEY is only used if BACKUP_PROG_CRYPT_ENABLED is true
+    # therefore 'Log ... BACKUP_PROG_CRYPT_KEY ...' is used (and not '$BACKUP_PROG_CRYPT_KEY')
+    # but '$BACKUP_PROG_CRYPT_KEY' must be used in the actual command call which means
+    # the BACKUP_PROG_CRYPT_KEY value would appear in the log when rear is run in debugscript mode
+    # so that stderr of the confidential command is redirected to /dev/null
+    # cf. the comment of the UserInput function in lib/_input-output-functions.sh
+    # how to keep things confidential when rear is run in debugscript mode
+    # because it is more important to not leak out user secrets into a log file
+    # than having stderr error messages when a confidential command fails
+    # cf. https://github.com/rear/rear/issues/2155
     (   case "$BACKUP_PROG" in
             (tar)
                 if [ -s $TMP_DIR/restore-exclude-list.txt ] ; then
-                    BACKUP_PROG_OPTIONS=( "${BACKUP_PROG_OPTIONS[@]}" "--exclude-from=$TMP_DIR/restore-exclude-list.txt" )
+                    BACKUP_PROG_OPTIONS+=( "--exclude-from=$TMP_DIR/restore-exclude-list.txt" )
+                fi
+                # Let 'dd' read and write up to 1M=1024*1024 bytes at a time to speed up things
+                # cf. https://github.com/rear/rear/issues/2369 and https://github.com/rear/rear/issues/2458
+                if is_true "$BACKUP_PROG_CRYPT_ENABLED" ; then 
+                    Log "dd if=$restore_input bs=1M | $BACKUP_PROG_DECRYPT_OPTIONS BACKUP_PROG_CRYPT_KEY | $BACKUP_PROG --block-number --totals --verbose ${BACKUP_PROG_OPTIONS[@]} ${BACKUP_PROG_COMPRESS_OPTIONS[@]} -C $TARGET_FS_ROOT/ -x -f -"
+                    dd if=$restore_input bs=1M | { $BACKUP_PROG_DECRYPT_OPTIONS "$BACKUP_PROG_CRYPT_KEY" ; } 2>/dev/null | $BACKUP_PROG --block-number --totals --verbose "${BACKUP_PROG_OPTIONS[@]}" "${BACKUP_PROG_COMPRESS_OPTIONS[@]}" -C $TARGET_FS_ROOT/ -x -f -
+
+                else
+                    Log "dd if=$restore_input bs=1M | $BACKUP_PROG --block-number --totals --verbose ${BACKUP_PROG_OPTIONS[@]} ${BACKUP_PROG_COMPRESS_OPTIONS[@]} -C $TARGET_FS_ROOT/ -x -f -"
+
+                    dd if=$restore_input bs=1M | $BACKUP_PROG --block-number --totals --verbose "${BACKUP_PROG_OPTIONS[@]}" "${BACKUP_PROG_COMPRESS_OPTIONS[@]}" -C $TARGET_FS_ROOT/ -x -f -
                 fi
-                Log dd if=$restore_input \| $BACKUP_PROG_DECRYPT_OPTIONS $BACKUP_PROG_CRYPT_KEY \| $BACKUP_PROG --block-number --totals --verbose "${BACKUP_PROG_OPTIONS[@]}" "${BACKUP_PROG_COMPRESS_OPTIONS[@]}" -C $TARGET_FS_ROOT/ -x -f -
-                dd if=$restore_input | $BACKUP_PROG_DECRYPT_OPTIONS $BACKUP_PROG_CRYPT_KEY | $BACKUP_PROG --block-number --totals --verbose "${BACKUP_PROG_OPTIONS[@]}" "${BACKUP_PROG_COMPRESS_OPTIONS[@]}" -C $TARGET_FS_ROOT/ -x -f -
                 ;;
             (rsync)
                 if [ -s $TMP_DIR/restore-exclude-list.txt ] ; then
-                    BACKUP_RSYNC_OPTIONS=( "${BACKUP_RSYNC_OPTIONS[@]}" --exclude-from=$TMP_DIR/restore-exclude-list.txt )
+                    BACKUP_RSYNC_OPTIONS+=( --exclude-from=$TMP_DIR/restore-exclude-list.txt )
                 fi
                 Log $BACKUP_PROG $v "${BACKUP_RSYNC_OPTIONS[@]}" "$restore_input"/ $TARGET_FS_ROOT/
                 $BACKUP_PROG $v "${BACKUP_RSYNC_OPTIONS[@]}" "$restore_input"/ $TARGET_FS_ROOT/
